SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[DeleteEvaluationResultDraftBackup] (
	@VCC VARCHAR(50) = NULL,
	@EvaluationPlanId VARCHAR(50) = NULL,
	@InteractionId VARCHAR(250) = NULL,
	@AuditorId VARCHAR(50) = NULL
) AS
BEGIN
	SET NOCOUNT ON
	-- EL PRESENTE SP TIENE LA FINALIDAD DE ELIMINAR LOS REGISTROS DE RESPALDO PARA BORRADOR

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
        
		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		-- DECLARAMOS UNA VARIABLE PARA VERIFICAR EL TIPO DEL PLAN
		DECLARE @IS_MIXED_EVALUATION_PLAN BIT = 0

		-- VERIFICAMOS QUE EL PLAN DE EVALUACION ES DEL TIPO MIXTO
		SELECT @IS_MIXED_EVALUATION_PLAN = IIF (EP.[Evaluation_Mode] = 'X', 1, 0) 
			FROM [dbo].[EvaluationPlan] EP (NOLOCK) 
			WHERE EP.[Instance] = @VCC
				AND EP.[ID] = @EvaluationPlanId

		-- DECLARAMOS UNA VARIABLE PARA VERIFICAR SI LA AUDITORIA YA ESTA GUARDADA COMO BORRADOR
		DECLARE @EXISTS_DRAFT_DATA BIT = 0

		IF EXISTS (
			SELECT 1 
				FROM [dbo].[EvaluationResult] (NOLOCK)
				WHERE [Instance] = @VCC
					AND [IdEvaluationPlan] = @EvaluationPlanId
					AND [IdInteraction] = @InteractionId
					AND [IdEvaluator] = 'SPEECH'
					AND [IdAuditor] = @AuditorId + '-DRAFT'
		)
			SET @EXISTS_DRAFT_DATA = 1

		IF @IS_MIXED_EVALUATION_PLAN = 1 AND @EXISTS_DRAFT_DATA = 1
		BEGIN
			-- RESTAURAMOS LOS RESULTADOS DE LA PARTE AUTOMATICA
			DELETE [dbo].[EvaluationResult]
				WHERE [Instance] = @VCC
					AND [IdEvaluationPlan] = @EvaluationPlanId
					AND [IdInteraction] = @InteractionId
					AND [IdEvaluator] = 'SPEECH'
					AND [IdAuditor] = @AuditorId + '-DRAFT'
		END 

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH
	
	
    SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
