SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[FinishedBatchClose](
	@VCC VARCHAR(50) = NULL,
	@BatchTypeName VARCHAR(50) = NULL,
	@BatchTypeId VARCHAR(50) = NULL
)
AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR LOS SALDOS DE LAS CARTERAS DEL VCC
	SET NOCOUNT ON

	-- VERIFICAMOS QUE TENEMOS LOS PARAMETROS NECESARIOS
	IF @VCC IS NULL OR @BatchTypeName IS NULL OR @BatchTypeId IS NULL
		-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
		RAISERROR('DATABABASE_ERROR_MISSING_PARAMETERS', 16, 16)

	-- DECLARAMOS UNA VARIABLE PARA EL ID DE AGENDA
		DECLARE @SCHEDULE_BATCH_ID VARCHAR(50)

	-- DECLARAMOS UNA VARIABLE PARA CONTABILIZAR LAS EJECUCIONES AUN NO TERMINADAS
	DECLARE @SCHEDULE_EXECUTIONS INT = 0

	--SELECT * FROM [dbo].[Batch] 
	UPDATE [dbo].[Batch] 
		SET [FinishReported] = 1
			,[LastModifiedDate] = GETUTCDATE()
		WHERE [VCC] = @VCC
			AND [BatchTypeName] = @BatchTypeName 
			AND [BatchTypeId] = @BatchTypeId

	-- SEGUN EL BATCHTYPENAME ACTUALIZAMOS EL STATUS DE LA TABLA DEL BATCH
	IF @BatchTypeName = 'TRANSCRIPTION'
	BEGIN
		UPDATE [dbo].[Process] 
			SET [Status] = CASE WHEN ([Total] - ISNULL([Error], 0)) > ISNULL([Error], 0) THEN 'COMPLETE'
							WHEN ([Total] - ISNULL([Error], 0)) < ISNULL([Error], 0) THEN 'FAILED' END
				,[StatusDescription] = CASE WHEN ISNULL([Error], 0) > 0 THEN 'COMPLETED_WITH_ERROR' 
										WHEN ISNULL([Error], 0) = 0 THEN 'COMPLETED' END
			WHERE [VCC] = @VCC
				AND [Name] = @BatchTypeId
				AND [Percentage] = 1

		-- OBTENEMOS EL ID DE LA AGENDA EN CASO EXISTA
		SELECT @SCHEDULE_BATCH_ID = P.[BatchScheduleId]
			FROM [dbo].[Process] P (NOLOCK)
			WHERE P.[VCC] =  @VCC
				AND P.[Name] = @BatchTypeId

		IF @SCHEDULE_BATCH_ID IS NOT NULL
		BEGIN
			-- ACTUALIZAMOS LA EJECUCION EN CASO SEA UN PROCESS AGENDADO
			UPDATE BSE 
				SET BSE.[Status] = P.[StatusDescription],
					BSE.[LastModifiedByUserId] = 'system',
					BSE.[LastModifiedDate] = GETUTCDATE()
				FROM [dbo].[BatchScheduleExecution] BSE (NOLOCK)
				INNER JOIN [dbo].[Process] P (NOLOCK)
					ON P.[VCC] = BSE.[VCC]
						AND P.[Name] = BSE.[BatchId]
				WHERE BSE.[VCC] = @VCC
					AND P.[Name] = @BatchTypeId
					AND P.[BatchScheduleId] IS NOT NULL
			
			SELECT @SCHEDULE_EXECUTIONS = COUNT(BSE.[Id]) 
				FROM [dbo].[BatchScheduleExecution] BSE (NOLOCK)
				INNER JOIN [dbo].[BatchSchedule] BS (NOLOCK)
					ON BS.[VCC] = BSE.[VCC]
						AND BS.[Id] = BSE.[BatchScheduleId]
				WHERE BSE.[VCC] = @VCC
					AND BS.[BatchType] = 'TRANSCRIPTION'
					AND BSE.[BatchScheduleId] = @SCHEDULE_BATCH_ID
					AND BSE.[Status] IN ('PENDING', 'REGISTERED_ON_REDIS', 'BATCH_CREATED', 'BATCH_STARTED')

			-- SI TENEMOS 0 EJECUCIONES NO TERMINADAS, DAMOS POR FINALIZADA LA AGENDA
			IF @SCHEDULE_EXECUTIONS = 0
				UPDATE [dbo].[BatchSchedule]
					SET [Status] = 'FINISHED',
						[LastModifiedByUserId] = 'system',
						[LastModifiedDate] = GETUTCDATE()
					WHERE [VCC] = @VCC 
						AND [ID] = @SCHEDULE_BATCH_ID
		END
	END
	ELSE IF @BatchTypeName IN ('STUDY', 'TEMPLATE')
	BEGIN 
		UPDATE [dbo].[Study] 
			SET [StatusDescription] = CASE WHEN [Percentage] = 1 AND  ISNULL([Error],0) > 0 THEN 'COMPLETED_WITH_ERROR' 
										WHEN [Percentage] = 1 AND  ISNULL([Error],0) = 0 THEN 'COMPLETED' END
			WHERE [VCC] = @VCC
				AND [Name] = @BatchTypeId
				AND [Percentage] = 1

		-- OBTENEMOS EL ID DE LA AGENDA EN CASO EXISTA
		SELECT @SCHEDULE_BATCH_ID = S.[BatchScheduleId]
			FROM [dbo].[Study] S (NOLOCK)
			WHERE S.[VCC] =  @VCC
				AND S.[Name] = @BatchTypeId

		IF @SCHEDULE_BATCH_ID IS NOT NULL
		BEGIN
			-- ACTUALIZAMOS LA EJECUCION EN CASO SEA UN PROCESS AGENDADO 
			UPDATE BSE 
				SET BSE.[Status] = S.[StatusDescription],
					BSE.[LastModifiedByUserId] = 'system',
					BSE.[LastModifiedDate] = GETUTCDATE()
				FROM [dbo].[BatchScheduleExecution] BSE (NOLOCK)
				INNER JOIN [dbo].[Study] S (NOLOCK)
					ON S.[VCC] = BSE.[VCC]
						AND S.[Name] = BSE.[BatchId]
				WHERE BSE.[VCC] = @VCC
					AND S.[Name] = @BatchTypeId
					AND S.[BatchScheduleId] IS NOT NULL
			
			SELECT @SCHEDULE_EXECUTIONS = COUNT(BSE.[Id]) 
				FROM [dbo].[BatchScheduleExecution] BSE (NOLOCK)
				INNER JOIN [dbo].[BatchSchedule] BS (NOLOCK)
					ON BS.[VCC] = BSE.[VCC]
						AND BS.[Id] = BSE.[BatchScheduleId]
				WHERE BSE.[VCC] = @VCC
					AND BS.[BatchType] = 'TEMPLATE'
					AND BSE.[BatchScheduleId] = @SCHEDULE_BATCH_ID
					AND BSE.[Status] IN ('PENDING', 'REGISTERED_ON_REDIS', 'BATCH_CREATED', 'BATCH_STARTED')

			-- SI TENEMOS 0 EJECUCIONES NO TERMINADAS, DAMOS POR FINALIZADA LA AGENDA
			IF @SCHEDULE_EXECUTIONS = 0
				UPDATE [dbo].[BatchSchedule]
					SET [Status] = 'RUNNING',
						[LastModifiedByUserId] = 'system',
						[LastModifiedDate] = GETUTCDATE()
					WHERE [VCC] = @VCC 
						AND [ID] = @SCHEDULE_BATCH_ID
		END
	END
	ELSE IF @BatchTypeName = 'EVALUATION_PLAN'
	BEGIN 
		UPDATE [dbo].[EvaluationPlan] 
			SET [StatusDescription] = CASE WHEN [Percentage] = 1 AND  ISNULL([Error], 0) > 0 THEN 'COMPLETED_WITH_ERROR' 
									WHEN [Percentage] = 1 AND  ISNULL([Error], 0) = 0 THEN 'COMPLETED' END 
			WHERE [Instance] = @VCC
				AND [ID] = @BatchTypeId
				AND [Percentage] = 1
				AND ([Status] = 3 OR [Status] = 5 OR [Status] = 7)
	END

	SELECT 1 'result'

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
