SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAllegroInteractions] (
	@Ip						VARCHAR(50),
	@VCC					VARCHAR(50),
	@StartDate				DATETIME = NULL,
	@EndDate				DATETIME = NULL,
	@StartHour				VARCHAR(50),
	@EndHour				VARCHAR(50),
	@Campaign				VARCHAR(MAX),
	@Agent					VARCHAR(MAX),
	@ResultCode				VARCHAR(MAX),
	@Type					INT,
	@Origin					VARCHAR(MAX),
	@MinDuration			INT,
	@MaxDuration			INT,
	@Format					VARCHAR(50) = NULL, 
	@InteractionIds         VARCHAR(MAX) = NULL, 
	@Evaluated              VARCHAR(MAX) = NULL,
	-- @UniqueIds				BIT = 0,							-- ESTE FLAG TIENE COMO PROPOSITO RETORNAR LA LISTA CONSOLIDAD POR IDS O EL TOTAL DEL LISTADO (AUDIOS/INTERACTIONS)
	@ResultType				VARCHAR(50) = NULL,						-- VARIABLE PARA DETERMINAR SI LA BUSQUEDA SERA SOBRE SOBRE DETALLES O ID (DETAILED, SUMMARIZED)
	@ContactAddress			VARCHAR(300) = NULL
) AS
BEGIN
	-- SETTEAMOS UN VALOR POR DEFECTO AL FORMAT
	IF @Format IS NULL OR @Format = '' SET @Format = 'mp3,wav'

	-- SETTEAMOS VALORES MINIMOS PARA LAS HORAS DE INICIO Y FIN
	IF (@StartHour IS NOT NULL AND @StartHour = '') SET @StartHour = '00:00:00'
	IF (@EndHour IS NOT NULL AND @EndHour = '') SET @EndHour = '23:59:59'

	-- DECLARAMOS UNA VARIABLE PARA IDENTIFICAR SI EL SP ES LLAMADO DEL SISTEMA O POR OTRO SP
	DECLARE @IS_NESTED BIT = 0

	-- CREAMOS LAS VARIABLES QUE HARAN REFERENCIA A LAS TABLAS QUE PUEDEN SER EXTERNAS
	DECLARE @TABLE_HISTORICAL_DATA VARCHAR(50) = ''
	DECLARE @TABLE_MMPRODAT VARCHAR(50) = ''

	-- DECLARAMOS CADENAS PARA EL QUERY FINAL
	DECLARE @SQL_TMP VARCHAR(MAX) = ''
	DECLARE @SQL_CTE VARCHAR(MAX) = ''
	DECLARE @SQL_CTE_SELECT VARCHAR(MAX) = ''
	DECLARE @SQL_CTE_FROM_FIXED VARCHAR(MAX) = ''
	DECLARE @SQL_CTE_FROM_OPTIONAL VARCHAR(MAX) = ''
	DECLARE @SQL_CTE_FILTERS_FIXED VARCHAR(MAX) = ''
	DECLARE @SQL_CTE_FILTERS_OPTIONAL VARCHAR(MAX) = ''
	DECLARE @SQL_CTE_FILTERS_EXCLUDING VARCHAR(MAX) = ''
	
	DECLARE @Sql NVARCHAR(MAX)

	-- EL TYPE 0 INDICA QUE LAS TABLAS SE ENCUENTRAN EN UN SERVIDOR EXTERNO
	IF(@Type = 0)  
	BEGIN  
		SET @TABLE_HISTORICAL_DATA = '[' + @Ip + ']' + '.[HistoricalData].[dbo].'
		SET @TABLE_MMPRODAT = '[' + @Ip + ']' + '.[MMProDat].[dbo].'
	END
	ELSE IF(@Type = 1) 
	BEGIN
		SET @TABLE_HISTORICAL_DATA = '[HistoricalData].[dbo].'
		SET @TABLE_MMPRODAT = '[MMProDat].[dbo].'
	END
	
	IF(@Type = 0 OR @Type = 1)  
	BEGIN  
		-- VERIFICAMOS SI EXISTE LA TABLA TEMPORAL #TMP_HD_INTERACTIONS
		IF OBJECT_ID('TEMPDB..#TMP_HD_INTERACTIONS') IS NULL
		BEGIN
			-- CREAMOS UNA TABLA TEMPORAL PARA INSERTAR LOS REGISTROS DE LA HDI6
			CREATE TABLE #TMP_HD_INTERACTIONS
			(
				[ID] VARCHAR(150),
				[InteractionId] VARCHAR(100),
				[Section] INT,
				[Part] INT,
				[Channel] VARCHAR(50),
				[Number] VARCHAR(100),
				[Gender] VARCHAR(50),
				[SilenceTime] INT,
				[DurationTime] INT,
				[AttentionTime] INT,
				[Category1] VARCHAR(100),
				[Category2] VARCHAR(100),
				[Category3] VARCHAR(100),
				[Description] VARCHAR(150),
				[DispositionCode] INT,
				[Campaign] VARCHAR(150),
				[Agent] VARCHAR(100),
				[AgentName] VARCHAR(100),
				[Type] VARCHAR(100),
				[StartDate] VARCHAR(100),
				[EndDate] VARCHAR(100),
				[Format] VARCHAR(50),
				[ContactAddress] VARCHAR(300),
				[ContactId]  VARCHAR(50),
				[ContactName] VARCHAR(300),
				[RelativeFileName] VARCHAR(1000),
				[RNA] INT
			)

			-- REAFIRMAMOS QUE EL SP NO ESTA ANIDADO EN OTRO
			SET @IS_NESTED = 0
		END
		ELSE
		BEGIN
			-- SI LA TABLA TEMPORAL YA EXISTIA, EL SP ESTA ANIDADO EN OTRO (P.E.: [dbo].[GetAsyncBillingInteractions])
			SET @IS_NESTED = 1
		END 

		-- INICIAMOS ARMANDO LOS FILTROS PARA LA TABLA CTE PRINCIPAL
		SET @SQL_CTE_FILTERS_FIXED = '
				WHERE ID.[VirtualCC] = ''' + @VCC + '''
					AND ID.[EndDate] BETWEEN ''' + (SELECT CONVERT(VARCHAR(21), @StartDate, 127)) + ''' 
					AND ''' + (SELECT CONVERT(VARCHAR(21), @EndDate, 127)) + '''
					AND IMP.[Format] IN (SELECT Value FROM [dbo].[split](''' + @Format + ''','','')) '

		-- AGREGAMOS EL SCRIPT PARA LOS FILREOS DE HORA MAXIMA Y HORA MINIMA LAS CUALES TIENEN UN COMPORTAMIENTO DIFERENTES SEGUN CUAL ES MAYOR
		IF (CAST(@StartHour AS TIME) <= CAST(@EndHour AS TIME)) 
			SET @SQL_CTE_FILTERS_OPTIONAL += '
					AND CAST(ID.[EndDate] AS TIME) BETWEEN ''' + @StartHour + ''' AND ''' + @EndHour + ''' '
		IF (CAST(@StartHour AS TIME) > CAST(@EndHour AS TIME)) 
			SET @SQL_CTE_FILTERS_OPTIONAL += '
					AND CAST(ID.[EndDate] AS TIME) NOT BETWEEN ''' + @EndHour + ''' AND ''' + @StartHour + ''' '

		-- AGREGAMOS EL SCRIPT PARA EL FILTRO DE CAMPAIGN
		IF (@Campaign IS NOT NULL AND @Campaign <> '') 
		BEGIN
			-- DEBEMOS PASAR LA LISTA DE CAMPAIGNS A UNA TABLA TEMPORAL PARA AGILIZAR LA BUSQUEDA 
			SET @SQL_TMP += '
			SELECT * 
				INTO #FILTER_CAMPAIGNS
				FROM (
					SELECT Value FROM [dbo].[split](''' + @Campaign + ''','','')
				) A '

			-- AGREGAMOS EL JOIN EN LA BUSQUEDA PRINCIPAL
			SET @SQL_CTE_FROM_OPTIONAL += '  
				INNER JOIN #FILTER_CAMPAIGNS FC 
					ON FC.[Value] = ID.[Campaign] ' 
		END

		-- AGREGAMOS EL SCRIPT PARA EL FILTRO DE AGENT
		IF (@Agent IS NOT NULL AND @Agent <> '') 
		BEGIN
			-- DEBEMOS PASAR LA LISTA DE AGENTS A UNA TABLA TEMPORAL PARA AGILIZAR LA BUSQUEDA 
			SET @SQL_TMP += '
			SELECT * 
				INTO #FILTER_AGENTS
				FROM (
					SELECT Value FROM [dbo].[split](''' + @Agent + ''','','')
				) A '

			-- AGREGAMOS EL JOIN EN LA BUSQUEDA PRINCIPAL
			SET @SQL_CTE_FROM_OPTIONAL += '  
				INNER JOIN #FILTER_AGENTS FA 
					ON FA.[Value] = ISNULL(ID.[LastAgent], ID.[FirstAgent]) '
		END

		-- AGREGAMOS EL SCRPT PARA EL FILTRO ORIGEN
		IF (@Origin IS NOT NULL AND @Origin <> '') 
			SET @SQL_CTE_FILTERS_OPTIONAL += '
				AND ID.[Direction] IN (SELECT Value FROM [dbo].[split](''' + @Origin + ''','','')) '

		-- AGREGAMOS EL SCRIPT PARA LOS FILTROS DE DURACION MAXIMA Y HORA MINIMA LAS CUALES TIENEN UN COMPORTAMIENTO DIFERENTES SEGUN CUAL ES MAYOR
		IF (@MinDuration IS NOT NULL) 
			SET @SQL_CTE_FILTERS_OPTIONAL += '
				AND ID.[AttentionTime] >= ' + (SELECT CONVERT(varchar(10), @MinDuration)) + ' '
		IF (@MaxDuration IS NOT NULL) 
			SET @SQL_CTE_FILTERS_OPTIONAL += '
				AND ID.[AttentionTime] <= ' + (SELECT CONVERT(varchar(10), @MaxDuration)) + ' '

		-- AGREGAMOS EL SCRIPT PARA EL FILTRO CONTACT
		IF (@ContactAddress IS NOT NULL)
			SET @SQL_CTE_FILTERS_OPTIONAL += '
				AND ID.[ContactId] LIKE ''%' + @ContactAddress + '%'' '

		-- AGREGAMOS EL SCRIPT PARA INTERACTION_ID
		IF @InteractionIds IS NOT NULL AND @InteractionIds <> ''
		BEGIN
			SET @SQL_TMP = ''
			SET @SQL_CTE_FROM_OPTIONAL = ''
			SET @SQL_CTE_FILTERS_OPTIONAL = ''
			
			-- DEBEMOS PASAR LA LISTA DE CAMPAIGNS A UNA TABLA TEMPORAL PARA AGILIZAR LA BUSQUEDA 
			SET @SQL_TMP += '
			SELECT * 
				INTO #FILTER_INTERACTIONS_IDS
				FROM (
					SELECT Value FROM [dbo].[split](''' + @InteractionIds + ''','','')
				) A '
			-- AGREGAMOS EL JOIN EN LA BUSQUEDA PRINCIPAL
			SET @SQL_CTE_FROM_OPTIONAL += '  
				INNER JOIN #FILTER_INTERACTIONS_IDS FID 
					ON FID.[Value] = ID.[Id] ' 
			-- SET @SQL_CTE_FILTERS_EXCLUDING = ' AND ID.[Id] IN (SELECT Value FROM [dbo].[split](''' + @InteractionIds + ''','','')) '
		END 


		-- ARMAMOS EL QUERY PARA EL CONTENIDO DE LA CTE
		SET @SQL_CTE_SELECT = '
			;WITH CTE_INTERACTION AS (
				SELECT ID.[VirtualCC]																						''VCC'',
						ID.[Id] + ''_'' + CONVERT(VARCHAR(10), IMP.[Section]) + ''_'' + CONVERT(VARCHAR(10), IMP.[Part])	''ID'',       
						ID.[Id]																								''InteractionId'',       
						IMP.[Section]																						''Section'',       
						IMP.[Part]																							''Part'',       
						''CALL''																							''Channel'',       
						CASE WHEN ID.Direction = ''E'' THEN CID.CallerId ELSE CID.CalledId END								''Number'',       
						''''																								''Gender'',      
						ID.[HoldTime]																						''SilenceTime'',       
						ID.[AttentionTime]																					''DurationTime'', -- ESTO SE DEBE CORREGIR       
						ID.[AttentionTime]																					''AttentionTime'', -- SE AGREGO CAMPO PARA AMBOS EXISTAN       
						ID.[Campaign]																						''Campaign'',       
						ISNULL(ID.[LastAgent], ID.[FirstAgent])																''Agent'',   
						ID.[ManagementResult]																				''ManagementResult'',
						ID.[Direction]																						''Type'',       
						CONVERT(VARCHAR, ID.[StartDate], 120)																''StartDate'',       
						CONVERT(VARCHAR, ID.[EndDate], 120)																	''EndDate'',       
						IMP.[Format]																						''Format'',       
						CASE WHEN ID.[Direction] = ''E'' THEN CID.[CallerId] ELSE CID.[CalledId] END						''ContactAddress'',       
						CASE WHEN ID.[Direction] = ''E'' THEN CID.[CallerId] ELSE CID.[CalledId] END						''ContactId'',       
						ID.[ContactName]																					''ContactName'',       
						IMP.[RelativePath] + ''/'' + IMP.[FileName]															''RelativeFileName'',       
						RNA = ROW_NUMBER() OVER(PARTITION BY (ID.[Id] + ''_'' + CONVERT(VARCHAR(10), IMP.[Section]) + ''_'' + CONVERT(VARCHAR(10), IMP.Part)) ORDER BY ID.[Id]) -- ROW NUMBER AUDIO            
					FROM [HistoricalData].[dbo].[InteractionDetail] ID (NOLOCK)    
					INNER JOIN [HistoricalData].[dbo].[InteractionMultimediaParts] IMP (NOLOCK)      
						ON IMP.Id = ID.Id
					INNER JOIN [HistoricalData].[dbo].[CallInteractionDetail] CID (NOLOCK)        
						ON ID.Id = CID.Id '
					+ @SQL_CTE_FROM_OPTIONAL
					+ @SQL_CTE_FILTERS_FIXED
					+ @SQL_CTE_FILTERS_OPTIONAL
					+ @SQL_CTE_FILTERS_EXCLUDING + '
			)
			,CTE_WITH_IAD AS (
				SELECT CET_I.*,
						IAD.[ActorName]											 ''agentName'' 
					FROM CTE_INTERACTION CET_I
					LEFT JOIN [HistoricalData].[dbo].[InteractionActorDetail] IAD (NOLOCK)       
						ON IAD.[VirtualCC] = CET_I.[VCC]
							AND IAD.[Id] = CET_I.[InteractionId]        
							AND IAD.[Section] = CET_I.[Section]        
							AND IAD.[Part] = CET_I.[Part]
			)
			,CTE_WITH_CGRM AS (	
				SELECT CTE_C.*,
						ISNULL(CGRM.[Category1], '''')								''Category1'',       
						ISNULL(CGRM.[Category2], '''')								''Category2'',       
						ISNULL(CGRM.[Category3], '''')								''Category3'',       
						ISNULL(CGRM.[Description], '''')							''Description'',       
						ISNULL(CONVERT(VARCHAR(4), CGRM.[Code]), '''')				''DispositionCode'' 
				FROM CTE_WITH_IAD CTE_C 
				LEFT JOIN (       
					SELECT VCC, CampaignID, [Order], [Code],            
							[Description], Category1, Category2, Category3         
						FROM [MMProDat].[dbo].[CampaignGoalResultManagement] (NOLOCK)
						WHERE VCC = ''' + @VCC + '''
				) CGRM         
					ON CGRM.VCC = CTE_C.[VCC]         
						AND CGRM.CampaignId = CTE_C.[Campaign]
						AND CGRM.[Description] = CTE_C.[ManagementResult]'

		-- AGREGAMOS EL SCRIPT PARA RESULTADO DE GESTION
		IF (@ResultCode IS NOT NULL AND @ResultCode <> '') 
		BEGIN
			-- DEBEMOS PASAR LA LISTA DE CAMPAIGNS A UNA TABLA TEMPORAL PARA AGILIZAR LA BUSQUEDA 
			SET @SQL_TMP += '
			SELECT * 
				INTO #FILTER_RESULT_CODES
				FROM (
					SELECT Value FROM [dbo].[split](''' + @ResultCode + ''','','')
				) A '

			-- AGREGAMOS EL JOIN EN LA BUSQUEDA PRINCIPAL
			SET @SQL_CTE_SELECT += '  
				INNER JOIN #FILTER_RESULT_CODES FRC 
					ON (
						FRC.[Value] = CONCAT(CTE_C.[Campaign], ''_'', ISNULL(CONVERT(VARCHAR(4), CGRM.[Code]), ''''))
						OR
						FRC.[Value] = CTE_C.[ManagementResult] 
					) '
		END

		SET @SQL_CTE_SELECT += '
			) -- SELECT * FROM CTE_WITH_CGRM'

		-- ARMAMOS EL QUERY PRINCIPAL
		SET @SQL_CTE = @SQL_TMP + @SQL_CTE_SELECT

		-- SEGUN SI EL SP ES ANIDADO O NO INSERTAMOS EN LA TEMPORAL EXISTENTE O LA CREAMOS
		IF @IS_NESTED = 1 
			SET @SQL_CTE += '
			INSERT INTO #TMP_HD_INTERACTIONS'	
		SET @SQL_CTE += '
			SELECT CTE_C.[ID],
					CTE_C.[InteractionId],
					CTE_C.[Section],
					CTE_C.[Part],
					CTE_C.[Channel],
					CTE_C.[Number],
					CTE_C.[Gender],
					CTE_C.[SilenceTime],
					CTE_C.[DurationTime],
					CTE_C.[AttentionTime],
					CTE_C.[Category1],
					CTE_C.[Category2],
					CTE_C.[Category3],
					CTE_C.[Description],
					CTE_C.[DispositionCode],
					CTE_C.[Campaign],
					CTE_C.[Agent],
					CTE_C.[AgentName],
					CTE_C.[Type],
					CTE_C.[StartDate],
					CTE_C.[EndDate],
					CTE_C.[Format],
					CTE_C.[ContactAddress],
					CTE_C.[ContactId],
					CTE_C.[ContactName],
					CTE_C.[RelativeFileName],
					CTE_C.[RNA] '
		IF @IS_NESTED = 0 
			SET @SQL_CTE += '
				INTO #TMP_HD_INTERACTIONS '
		SET @SQL_CTE += '
				FROM CTE_WITH_CGRM CTE_C
			DROP TABLE IF EXISTS #FILTER_CAMPAIGNS 
			DROP TABLE IF EXISTS #FILTER_AGENTS
			DROP TABLE IF EXISTS #FILTER_RESULT_CODES
			DROP TABLE IF EXISTS #FILTER_INTERACTIONS_IDS '

		-- EJECUTAMOS EL SQL QUE NOS DEJA LA TEMPORAL #TMP_HD_INTERACTIONS CON REGISTROS
		--PRINT @SQL_CTE
		--EXEC (@SQL_CTE)
		--SELECT * FROM #TMP_HD_INTERACTIONS

		-- VARIABLES PARA TRABAJAR CON LA TABLA TEMPORAL AL FINAL DEL SP
		DECLARE @QUERY_SUMMARY VARCHAR(MAX) = ''
		DECLARE @QUERY_DETAIL VARCHAR(MAX) = ''
		DECLARE @QUERY VARCHAR(MAX) = ''
		DECLARE @QUERY_FILTER VARCHAR(MAX) = ''
		DECLARE @QUERY_JOIN VARCHAR(MAX) = ''
	
		-- FILTRO POR INTERACCIONES EVALUADAS O NO
		IF(@Evaluated IS NOT NULL AND @Evaluated <> '')
		BEGIN 
			DECLARE @EvaluatedTable TABLE (Val BIT)
			INSERT INTO @EvaluatedTable SELECT Value FROM [dbo].[split]( @Evaluated ,',')
			-- SOLAMENTE SI ES SI O NO
			IF((SELECT COUNT(Val) FROM @EvaluatedTable) = 1 )
			BEGIN
				IF((SELECT Val FROM @EvaluatedTable WHERE Val = 1) IS NOT NULL) -- Si  es Si
				BEGIN 
					SET @QUERY_JOIN = ' JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK) ON (AEP.[IdInteraction] = HDI.[InteractionId] AND AEP.[Completed] = 1) '
				END
				ELSE  --Si es No
				BEGIN
					SET @QUERY_JOIN += ' LEFT JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK) ON (AEP.[IdInteraction] = HDI.[InteractionId] AND AEP.[Completed] = 1) '
					SET @QUERY_FILTER = ' AND AEP.[IdInteraction] IS NULL '
				END
			END
		END
		-- FIN FILTRO INTERACCIONES EVALUADAS

		-- SI EL SP NO ESTA ANIDADO, HACEMOS SELECT AL RESULTADO A MOSTRAR
		IF @IS_NESTED = 0
		BEGIN
			-- EN AMBOS CASOS SE REQUIERE EL RESUMEN
			SET @QUERY_SUMMARY = @SQL_CTE + '
				SELECT HDI.[channel]										AS ''interactionType'',
						COUNT(DISTINCT (HDI.[InteractionId]))				AS ''interactions'',
						COUNT(DISTINCT (HDI.[ID]))							AS ''details'',
						SUM(HDI.[AttentionTime])							AS ''attentionTime''
					FROM #TMP_HD_INTERACTIONS HDI '
			SET @QUERY +=  @QUERY_SUMMARY + @QUERY_JOIN
			SET @QUERY += ' WHERE HDI.[RNA] = 1 ' + @QUERY_FILTER + ' GROUP BY HDI.[channel] '

			-- HACEMOS EL SELECT SEGUN EL TIPO DE BUSQUEDA SETTEADO
			--IF @ResultType = 'SUMMARIZED'
			--BEGIN
			-- END
			-- ELSE 
			IF @ResultType = 'DETAILED'
			BEGIN
				SET @QUERY_DETAIL = '
					SELECT HDI.* 
						FROM #TMP_HD_INTERACTIONS HDI ' 
				SET @QUERY += @QUERY_DETAIL + @QUERY_JOIN 
				SET @QUERY += ' WHERE HDI.[RNA] = 1 ' + @QUERY_FILTER + ' '
			END

			-- EJECUTAMOS LA CONSULTA
			PRINT @QUERY
			EXEC (@QUERY)
    
			-- ELIMINAMOS LA TABLA TEMPORARL
			DROP TABLE #TMP_HD_INTERACTIONS
		END

	END
	ELSE
	BEGIN
		SET @Sql = 'SELECT 1 AS ''result'''		
		EXEC (@Sql)
	END

	

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
