SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetAudiosTranscriptions]
( 
	@PageSize				INT,
	@PageNumber				INT,
	@VCC					VARCHAR(50),
	@ProcessId				VARCHAR(50),
	@ID						VARCHAR(50),
	@StartDate				DATETIME = NULL,
	@EndDate				DATETIME = NULL,
	@StartHour				INT,
	@StartMinute			INT,
	@EndHour				INT,
	@EndMinute				INT,
	@Campaign				VARCHAR(MAX),
	@Agent					VARCHAR(MAX),
	@ResultCode				VARCHAR(MAX),
    @InteractionIds         VARCHAR(MAX) = NULL,
	@ContactAddress		    VARCHAR(300) =	NULL,
	@SentimentIds			VARCHAR(MAX) = NULL
)
AS
BEGIN

 
	DECLARE @SQL                    NVARCHAR(MAX);
    DECLARE @FILTERS_OPTIONAL_PART  VARCHAR(MAX) = ''
    DECLARE @FILTERS_EXCLUDING_PART VARCHAR(MAX) = ''
    DECLARE @FILTERS_FIXED_PART     VARCHAR(MAX) = ''

	SET @SQL = 'SET NOCOUNT ON
	DECLARE @rows INT
	SET @rows = (SELECT COUNT(A.ID)
		FROM [dbo].[Audio] A (NOLOCK)
		INNER JOIN [dbo].[ProcessAudio] PA (NOLOCK) 
			ON PA.[VCC] = A.[VCC]
				AND A.ID = PA.AudioId 
				AND A.Section = PA.Section 
				AND A.Part = PA.Part
		LEFT JOIN  SentimentAnalysisResult SAR 
			ON A.[VCC] = SAR.[VCC]
					AND A.[ID] = SAR.InteractionId 
					AND A.[Section] = SAR.InteractionSection 
					AND A.[Part] = SAR.InteractionPart
					AND ISNULL(A.[EndDate],0) = ISNULL(SAR.[EndDate],0)
					AND SAR.[AnalysisType] =' +''''+'general'+''''+ '
					AND SAR.[MaxScore] = 1
		LEFT JOIN [dbo].[Process] P (NOLOCK)
			-- ON P.[Description] = PA.[ProcessId]
			ON P.[VCC] = A.[VCC]
				AND P.[Name] = PA.[ProcessId]
		WHERE A.[VCC] = ''' + @VCC + ''' 
			AND A.[IsTranscribed] = 1 
			AND P.[TYPE] = ''SPEECH'' '
			IF(@ProcessId IS NOT NULL)
			BEGIN
				SET @SQL += '
				AND PA.[ProcessId]= ''' + @ProcessId + ''' '
			END
	
	BEGIN
		IF(@ID IS NOT NULL)
		BEGIN
			SET @FILTERS_OPTIONAL_PART += '
				AND CONCAT(A.[ID], ''_'', A.[Section], ''_'', A.[Part]) LIKE ''%' + @ID + '%''
			'
		END
		IF(@Campaign IS NOT NULL AND LEN(@Campaign) > 0)
		BEGIN
			SET @FILTERS_OPTIONAL_PART += '
				AND A.[Campaign] IN(''' + @Campaign + ''')
			'
		END
		IF(@Agent IS NOT NULL AND LEN(@Agent) > 0)
		BEGIN
			SET @FILTERS_OPTIONAL_PART += '
				AND A.[Agent] IN(''' + @Agent + ''')
			'
		END
		IF(@ResultCode IS NOT NULL AND LEN(@ResultCode) > 0)
		BEGIN
			SET @FILTERS_OPTIONAL_PART += '
				AND CONCAT(A.Campaign, ''_'', ISNULL(CONVERT(VARCHAR(4), A.[DispositionCode]), '''')) IN (''' + @ResultCode + ''')
			'
		END
		IF(@StartDate IS NOT NULL AND @EndDate IS NOT NULL)
		BEGIN
			SET @FILTERS_OPTIONAL_PART += '
				AND A.[EndDate] BETWEEN ''' + CONVERT(VARCHAR(10), @StartDate, 120) + ''' AND ''' + CONVERT(VARCHAR(10), @EndDate, 120) + '''
			'
		END
		IF(@StartHour IS NOT NULL AND @EndHour IS NOT NULL)
		BEGIN
			SET @FILTERS_OPTIONAL_PART += '
				AND DATEPART(HOUR, A.[EndDate]) >= ' + CONVERT(VARCHAR, @StartHour) + ' AND DATEPART(HOUR, A.[EndDate]) <= ' + CONVERT(VARCHAR, @EndHour) + '
			'
		END
		IF(@StartMinute IS NOT NULL AND @EndMinute IS NOT NULL)
		BEGIN
			SET @FILTERS_OPTIONAL_PART += '
				AND DATEPART(MINUTE, A.[EndDate]) >= ' + CONVERT(VARCHAR, @StartMinute) + ' AND DATEPART(MINUTE, A.[EndDate]) <= ' + CONVERT(VARCHAR, @EndMinute) + '
			'
		END
		IF(@ContactAddress IS NOT NULL)
		BEGIN
			SET @FILTERS_OPTIONAL_PART += '
				AND A.ContactAddress LIKE ''%' + @ContactAddress + '%''
			'
		END

	
		IF (@SentimentIds IS NOT NULL AND @SentimentIds <> '')
        BEGIN		
			
			SET @FILTERS_OPTIONAL_PART += ' AND 
						 SAR.[SentimentId] IN ('+''''+ @SentimentIds +''''+ ')						
						'
	    END
		
		IF (@InteractionIds IS NOT NULL AND @InteractionIds <> '')
        BEGIN 
            SET @FILTERS_OPTIONAL_PART = '';
	        SET @FILTERS_EXCLUDING_PART=' AND A.[ID] IN (SELECT Value FROM [dbo].[split](''' + @InteractionIds + ''','','')) '
        END

		
	
	END
	SET @SQL += @FILTERS_OPTIONAL_PART + @FILTERS_EXCLUDING_PART
	SET @SQL += ') '
	SET @SQL += ' SELECT @rows AS total '
	
	SET @SQL += '
		SELECT
			A.[ID]						''id'',
			A.[Section]					''section'',
			A.[Part]					''part'',
			PA.[ProcessId]				''processId'',
			A.[Number]					''number'',
			B.[Name]					''name'',
			A.[SilenceTime]				''silenceTime'',
			A.[DurationTime]			''durationTime'',
			A.[DurationLinguisticTime]	''durationLinguisticTime'',
			A.[Category1]				''category1'',
			A.[Category2]				''category2'',
			A.[Category3]				''category3'',
			A.[Agent]					''agent'',
			A.[AgentName]				''agentName'',
			A.[Description]				''description'',
			B.[Environment]				''environment'',
			E.[Type]					''environmentType'',
			A.[IsTranscribed]			''isTranscribed'',
			COUNT(UT.[Text])		    ''isSubrayed'',
			A.[CreatedDate]				''createdDate'',
			A.[EndDate]					''endDate'',
			SAR.[SentimentId]			''sentimentResult''
		FROM [dbo].[Audio] A (NOLOCK)
		INNER JOIN [dbo].[ProcessAudio] PA (NOLOCK) 
			ON PA.[VCC] = A.[VCC]
				AND A.ID = PA.AudioId 
				AND A.Section = PA.Section 
				AND A.Part = PA.Part
		LEFT JOIN  SentimentAnalysisResult SAR 
			ON A.[VCC] = SAR.[VCC]
					AND A.[ID] = SAR.InteractionId 
					AND A.[Section] = SAR.InteractionSection 
					AND A.[Part] = SAR.InteractionPart
					AND ISNULL(A.[EndDate],0) = ISNULL(SAR.[EndDate],0)
					AND SAR.[AnalysisType] =' +''''+'general'+''''+ '
					AND SAR.[MaxScore] = 1
		INNER JOIN [dbo].[Process] B (NOLOCK) 
			ON B.[VCC] = PA.[VCC]
				AND PA.[ProcessId] = B.[Name]
		INNER JOIN [dbo].[Environment] E (NOLOCK) 
			ON E.[VCC] = B.[VCC]
				AND B.[Environment] = E.[Name] 
				AND B.VCC = E.VCC
		LEFT JOIN [dbo].[Underlined] U (NOLOCK) 
			ON U.[VCC] = A.[VCC]
				AND A.ID = U.AudioId 
				AND A.Section = U.Section 
				AND A.Part = U.Part
	    LEFT JOIN [dbo].[Utterance] UT (NOLOCK)
	        ON UT.[VCC] = U.[VCC]
	           AND U.[UtteranceId] = UT.[Id]
		WHERE
			A.[VCC] = ''' + @VCC + ''' AND A.[IsTranscribed] = 1 '
			IF(@ProcessId IS NOT NULL)
			BEGIN
				SET @SQL += '
				AND PA.[ProcessId] = ''' + @ProcessId + ''' '
			END
			SET @SQL += '
	'

	SET @SQL += @FILTERS_OPTIONAL_PART + @FILTERS_EXCLUDING_PART
	
	SET @SQL += '
		AND B.[TYPE] = ''SPEECH''
		GROUP BY A.[ID], A.[Section], A.[Part], PA.[ProcessId], A.[Number],
				B.[Name], A.[SilenceTime], A.[DurationTime], A.[DurationLinguisticTime], A.[Category1],     
				A.[Category2], A.[Category3], A.[Agent], A.[AgentName], A.[Description], 
				B.[Environment], E.[Type], A.[IsTranscribed], A.[CreatedDate], A.[EndDate], SAR.[SentimentId]
		ORDER BY A.[ID], PA.[ProcessId], A.[Agent], A.[Description], B.[Environment], A.[DurationTime], A.[CreatedDate]
		OFFSET ' + CONVERT(VARCHAR(50), @PageSize) + ' * (' + CONVERT(VARCHAR(50), @PageNumber) + ' - 1) ROWS
		FETCH NEXT ' + CONVERT(VARCHAR(50), @PageSize) + ' ROWS ONLY;
	'
	
	SET @SQL += ' SET NOCOUNT OFF'	
	EXEC(@SQL)
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
