SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetAverageGroupRating] (
	@Instance VARCHAR(50) = NULL,
	@InitialDate DATETIME = NULL,
	@EndDate DATETIME = NULL,
	@IdCampaign VARCHAR(MAX) = 'GetAverageGroupRating',
	@IdEvaluationFormat VARCHAR(MAX) = 'GetAverageGroupRating',
	@IdAgent VARCHAR(MAX) = 'GetAverageGroupRating',
	@IdWorkGroup VARCHAR(MAX) = 'GetAverageGroupRating',
	@IdAttentionLevel VARCHAR(MAX) = 'GetAverageGroupRating',
	@IdEvaluationPlan VARCHAR(MAX) = 'GetAverageGroupRating',
	@IdGroup VARCHAR(MAX) = 'GetAverageGroupRating',
	@IdSubGroup VARCHAR(MAX) = 'GetAverageGroupRating',
	@IdAbility VARCHAR(MAX) = 'GetAverageGroupRating',
	@IdAuditor VARCHAR(MAX) = 'GetAverageGroupRating'
)
AS
BEGIN
	SET NOCOUNT ON;

	SELECT
		AverageGroupRating.*,
		(100 * AverageGroupRating.AverageRating / AverageGroupRating.MaxAbilityScoreAverage) AverageRatingNorm
	FROM
		(
			-- Grupo, grupo y subgrupo o grupo, subgrupo y habilidad con su promedio
			SELECT
				IdGroup
				,[Group].Description
				,CASE WHEN @IdGroup = 'GetAverageGroupRating' THEN '' ELSE IdSubGroup END AS IdSubGroup
				,[SubGroup].Description SubGroupDescription
				,CASE WHEN (@IdGroup = 'GetAverageGroupRating' OR @IdSubGroup = 'GetAverageGroupRating') THEN '' ELSE IdAbility END AS IdAbility
				,[Ability].Title AbilityTitle
				,AVG(InteractionTotalFilterQualification) AverageRating
				,AVG(MaxAbilityScore) MaxAbilityScoreAverage
			FROM
				(
					-- Llamadas evaluadas con la suma de los puntajes agrupados por grupo, grupo y subgrupo o grupo, subgrupo y habilidad
					SELECT
						ER.IdInteraction,
						ER.IdGroup,
						CASE WHEN @IdGroup = 'GetAverageGroupRating' THEN '' ELSE ER.IdSubGroup END AS IdSubGroup,
						CASE WHEN (@IdGroup = 'GetAverageGroupRating' OR @IdSubGroup = 'GetAverageGroupRating') THEN '' ELSE ER.IdAbility END AS IdAbility,
						SUM(ER.Qualification) InteractionTotalFilterQualification,
						SUM(ER.MaxAbilityScore) MaxAbilityScore
					FROM
						EvaluationResult ER (NOLOCK),
						(
							-- Llamadas evaluadas con su ultima fecha de evaluacion
							SELECT
								IdInteraction,
								MAX(EvaluationDate) LastEvaluationDate
							FROM 
								EvaluationResult (NOLOCK)
							WHERE
								EvaluationResult.Instance = @Instance
								AND EvaluationDate BETWEEN @InitialDate AND @EndDate
								AND CASE WHEN @IdAgent = 'GetAverageGroupRating' THEN 'GetAverageGroupRating' ELSE IdAgent END IN (SELECT Value FROM [dbo].[split](@IdAgent,','))
								AND CASE WHEN @IdCampaign = 'GetAverageGroupRating' THEN 'GetAverageGroupRating' ELSE IdCampaign END IN (SELECT Value FROM [dbo].[split](@IdCampaign,','))
								AND CASE WHEN @IdEvaluationFormat = 'GetAverageGroupRating' THEN 'GetAverageGroupRating' ELSE IdEvaluationFormat END IN (SELECT Value FROM [dbo].[split](@IdEvaluationFormat,','))
								AND CASE WHEN @IdEvaluationPlan = 'GetAverageGroupRating' THEN 'GetAverageGroupRating' ELSE IdEvaluationPlan END IN (SELECT Value FROM [dbo].[split](@IdEvaluationPlan,','))
							    AND IdAuditor LIKE CASE WHEN @IdAuditor = 'GetAverageGroupRating' THEN '%' ELSE 'SPEECH' END 
								AND IdAuditor <> CASE WHEN @IdAuditor = 'GetAverageGroupRating' THEN 'SPEECH' ELSE 'B_I_G_C_H_E_E_S_E' END
							GROUP BY IdInteraction
						) LastEvaluation
					WHERE
						ER.IdInteraction = LastEvaluation.IdInteraction
						AND ER.EvaluationDate = LastEvaluation.LastEvaluationDate
						AND CASE WHEN @IdGroup = 'GetAverageGroupRating' THEN 'GetAverageGroupRating' ELSE IdGroup END IN (SELECT Value FROM [dbo].[split](@IdGroup,','))
						AND CASE WHEN @IdSubGroup = 'GetAverageGroupRating' THEN 'GetAverageGroupRating' ELSE IdSubGroup END IN (SELECT Value FROM [dbo].[split](@IdSubGroup,','))
						AND CASE WHEN @IdAbility = 'GetAverageGroupRating' THEN 'GetAverageGroupRating' ELSE IdAbility END IN (SELECT Value FROM [dbo].[split](@IdAbility,','))
					    AND IdAuditor LIKE CASE WHEN @IdAuditor = 'GetAverageGroupRating' THEN '%' ELSE 'SPEECH' END AND IdAuditor <> CASE WHEN @IdAuditor = 'GetAverageGroupRating' THEN 'SPEECH' ELSE 'B_I_G_C_H_E_E_S_E' END
					GROUP BY
						ER.IdGroup,
						CASE WHEN @IdGroup = 'GetAverageGroupRating' THEN '' ELSE ER.IdSubGroup END,
						CASE WHEN (@IdGroup = 'GetAverageGroupRating' OR @IdSubGroup = 'GetAverageGroupRating') THEN '' ELSE ER.IdAbility END,
						ER.IdInteraction
				) InteractionTotalFilterQualification
				LEFT JOIN [Group] (NOLOCK) ON InteractionTotalFilterQualification.IdGroup = [Group].ID
				LEFT JOIN [SubGroup] (NOLOCK) ON InteractionTotalFilterQualification.IdSubGroup = [SubGroup].ID
				LEFT JOIN [Ability] (NOLOCK) ON InteractionTotalFilterQualification.IdAbility = [Ability].ID
			GROUP BY
				IdGroup
				,[Group].Description
				,CASE WHEN @IdGroup = 'GetAverageGroupRating' THEN '' ELSE IdSubGroup END
				,[SubGroup].Description
				,CASE WHEN (@IdGroup = 'GetAverageGroupRating' OR @IdSubGroup = 'GetAverageGroupRating') THEN '' ELSE IdAbility END
				,[Ability].Title
			) AverageGroupRating

	SET NOCOUNT OFF
END

GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
