SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetBillingInteractions] (
	@VCC					VARCHAR(50) = NULL,
	@SecondsPerRequest		DECIMAL(18, 5) = 0,
	@CharsPerPhrase			DECIMAL(18, 5) = 0,
	@Channel				VARCHAR(25) = NULL,
	@Retranscription		BIT = 0
	
)
AS
BEGIN
	
	-- SEGUN EL TIPO DE CANAL DEFINIMOS LA FUNCION
	IF @Channel = 'CALL'
	BEGIN
		-- DECLARAMOASUN CADENA PARA EL QUERY FINAL
		DECLARE @VCC_TYPE VARCHAR(50) = ''
		
		-- RECUPERAMOS EL TIPO DE VCC
		SELECT @VCC_TYPE = V.[Type] FROM [dbo].[VCC] V (NOLOCK) WHERE V.[VCC] = @VCC

		-- CREAMOS UNA TEMPORAL PARA AUDIO CON SU DEFINICION
		CREATE TABLE #INNER_AUDIO (
			[VCC] VARCHAR(50),
			[ID] VARCHAR(100),
			[Section] NUMERIC(5,0),
			[Part] NUMERIC(5,0), 
			[DurationTime] NUMERIC(12,0), 
			[TotalPhrases] INT,
			[IsProcess] BIT,
			[IsTranscribed] BIT,
			[IsCancel] BIT
		)
		
		-- SEGUN EL TIPO DE VCC COMPLETAMOS EL FROM
		IF @VCC_TYPE <> 'I6' -- I6
		BEGIN
			--OBTENEMOS TODOS AUDIOS RELACIONADOS A LA BUSQUEDA
			INSERT INTO #INNER_AUDIO
				SELECT DISTINCT A.[VCC],
								A.[ID],
								A.[Section],
								A.[Part],
								A.[DurationTime],
								A.[TotalPhrases],
								A.[IsProcess],
								A.[IsTranscribed],
								A.[IsCancel]
					FROM [dbo].[Audio] A (NOLOCK)
					WHERE A.[VCC] = @VCC
		END
		ELSE
	    BEGIN
			-- GENERAMOS EL 1ER RESUMEN DE AUDIOS
			;WITH CTE_AUDIO AS (
				SELECT DISTINCT A.[VCC],
								A.[ID],
								A.[Section],
								A.[Part],
								A.[DurationTime],
								A.[TotalPhrases],
								A.[IsProcess],
								A.[IsTranscribed],
								A.[IsCancel],
								RN = ROW_NUMBER() OVER(PARTITION BY A.[ID] ORDER BY A.[EndDate] DESC)
					FROM [dbo].[Audio] A (NOLOCK)
					WHERE A.[VCC] = @VCC
			)

			--OBTENEMOS TODOS AUDIOS RELACIONADOS A LA BUSQUEDA
			INSERT INTO #INNER_AUDIO
				SELECT CTE.[VCC],
						CTE.[ID],
						CTE.[Section],
						CTE.[Part],
						CTE.[DurationTime],
						CTE.[TotalPhrases],
						CTE.[IsProcess],
						CTE.[IsTranscribed],
						CTE.[IsCancel] 
					FROM CTE_AUDIO CTE
					WHERE CTE.[RN] = 1
				
		END
		
		-- HACEMOS LA BUSQUEDA
		SELECT SUM(CASE WHEN A.[IsTranscribed] = 1 
								AND @Retranscription = 0 
							THEN 1 ELSE 0 END)															'alreadyTranscribedAudio',
				SUM(CASE WHEN A.[IsTranscribed] = 1 
								AND @Retranscription = 0 
							THEN A.[DurationTime] ELSE 0 END)											'alreadyTranscribedDurationTime',
				SUM(CASE WHEN (A.[IsProcess] = 1 
								AND A.[IsTranscribed] IS NULL 
								AND A.[IsCancel] IS NULL) 
							THEN 1 ELSE 0 END)															'inProcessAudio',
				SUM(CASE WHEN (A.[IsProcess] = 1 
								AND A.[IsTranscribed] IS NULL 
								AND A.[IsCancel] IS NULL) 
							THEN A.[DurationTime] ELSE 0 END)											'inProcessDurationTime',
				'CALL'																				'interactionType',
				SUM(CASE WHEN (
					A.[IsProcess] IS NULL 
					OR
					@Retranscription = 1
					OR 
					(
						A.[IsProcess] = 1 
						AND 
						A.[IsCancel] = 1
					)
				) THEN 1 ELSE 0 END)																	'notTranscribedAudio',
				SUM(CASE WHEN (
					A.[IsProcess] IS NULL 
					OR
					@Retranscription = 1
					OR (
						A.[IsProcess] = 1 
						AND 
						A.[IsCancel] = 1
					)
				--) THEN A.[DurationTime] ELSE 0 END)														'notTranscribedDurationTime',
				) THEN IT.[AttentionTime] ELSE 0 END)														'notTranscribedDurationTime',
				SUM(CASE WHEN A.[IsTranscribed] = 1  
								AND A.[TotalPhrases] IS NOT NULL
							THEN A.[TotalPhrases] ELSE 0 END)											'totalPhrases',
				SUM(CASE WHEN (A.[IsProcess] IS NULL 
								OR (A.[IsProcess] = 1 
									AND A.[IsCancel] = 1)
								OR (A.[TotalPhrases] IS NULL)) 
							THEN (IT.[AttentionTime] / @SecondsPerRequest) ELSE 0 END)					'totalRawPhrases',
				SUM(IT.[AttentionTime] / @SecondsPerRequest)											'averageModelRequest' 
			FROM [#InteractionsTemp] IT
			INNER JOIN [dbo].[VCC] V (NOLOCK)
				ON V.[VCC] = IT.[VCC]
			LEFT JOIN [#INNER_AUDIO] A 
				ON A.[VCC] = IT.[VCC]
					AND  A.[ID] = IT.[InteractionId]
					AND (
						(
							V.[Type] <> 'I6'
							AND 
							A.[Section] = IT.[Section]
							AND 
							A.[Part] = IT.[Part]
						)
						OR 
						(
							V.[Type] = 'I6'
						)
					)
			WHERE IT.[VCC] = @VCC

		-- ELIMINAMOS LA TABLA TEMPORAL
		DROP TABLE IF EXISTS #INNER_AUDIO
	END
	ELSE
	BEGIN
		-- DECLARAMOS UNA VARIABLE PARA PARA EL TOTALDE RESULTADOS
		DECLARE @TOTAL_INTERACTIONS INT = 0

		-- SETTEAMOS EL NUMERO DE REGISTROS A LA VARIABLE DE TOTAL
		SELECT @TOTAL_INTERACTIONS = COUNT(*) FROM [#InteractionsTemp]

		IF @TOTAL_INTERACTIONS = 0 
		BEGIN
			-- SI NO EXISTEN RESULTADOS EN HD LLENAMOSUN RESULTADOS CON 0
			SELECT @Channel																					'interactionType',
					0																						'totalInteractionDetails',
					0																						'alreadyTranscribedInteraction',
					0																						'inProcessInteraction',
					0																						'notTranscribedInteraction',
					0																						'totalPhrases',
					0																						'totalRawChars',
					0																						'totalRawPhrases'
		END 
		ELSE
		BEGIN
			-- CRUZAMOS LA TABLA TEMPORAL DE INTERACCIONES ASINCRONAS CON LA TABLA LOCAL INTERACTION PARA LAS INTERACCIONES ASINCRONAS YA REGISTRADAS
			SELECT ISNULL(IT.[Channel], @Channel)															'interactionType',
					COUNT(*)																				'totalInteractionDetails',
					SUM(CASE WHEN (AI.[VCC] IS NOT NULL 
						AND AI.[IsTranscribed] = 1) THEN 1 ELSE 0 END)										'alreadyTranscribedInteraction',
					0																						'inProcessInteraction',
					SUM(CASE WHEN (AI.[VCC] IS NULL 
						OR AI.[IsTranscribed] <> 1) THEN 1 ELSE 0 END)										'notTranscribedInteraction',
					SUM(ISNULL(AI.[TotalPhrases], 0))														'totalPhrases',
					-- CUANDO TENEMOS VALIDPHRASES, YA NO CONTAMOS LAS RAWCHAR 
					SUM(
						CASE WHEN (AI.[TotalPhrases] IS NULL 
								OR AI.[TotalPhrases] = 0)
						THEN (
							ISNULL(AI.[RawChar], ISNULL(IT.[rawChar], 0))
						)
						ELSE 0 END
					)																						'totalRawChars',
					SUM(
						CASE WHEN (AI.[TotalPhrases] IS NULL 
								OR AI.[TotalPhrases] = 0)
						THEN (
							-- SI RAWCHAR ES NULL, SE DEBE VALUAR COMO MINIMO 1 FRASE(@CharsPerPhrase)
							-- CEILING(ISNULL(AI.[RawChar], ISNULL(HDI.[rawChar], @CharsPerPhrase)) * 1.00 / @CharsPerPhrase)
							CEILING(ISNULL(AI.[RawChar], ISNULL(IT.[RawChar], 0)) * 1.00 / @CharsPerPhrase)
						)
						ELSE 0 END
					)																						'totalRawPhrases' 
				FROM [#InteractionsTemp] IT
				LEFT JOIN [dbo].[AsyncInteraction] AI 
					ON IT.[VCC] = AI.[VCC]
						AND IT.[Channel] = AI.[Channel]
						--AND IT.[Campaign] = AI.[Campaign]
						AND IT.[InteractionId] = AI.[ID]
						AND IT.[EndDate] = AI.[EndDate]
					--WHERE AI.[VCC] = @VCC
				GROUP BY IT.[VCC],
							IT.[Channel]

		END
	END

	-- ELIMINAMOS LA TABLA TEMPORAL
	DROP TABLE [#InteractionsTemp] 
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
