SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[GetEvaluationFormats] (
	@Instance Varchar(50) = NULL, 
	@SearchCriteria Varchar(50) = NULL, 
	@SearchExpression Varchar(50) = NULL, 
	@Disabled bit = NULL
) AS
BEGIN
	SET NOCOUNT ON;

	-- CREAMOS LAS CTE QUE USAREMOS PARA RECUPERAR LOS MODELOS DE LOS FORMATOS
	WITH CTE_FORMAT_NLUS AS (
		SELECT DISTINCT EF.[Instance]
						, EF.[TemplateId]
						, N.[AppID]
						, N.[Name]
						, N.[Culture]
			FROM [dbo].[EvaluationFormat] EF (NOLOCK)
			INNER JOIN [dbo].[AbilityEvaluationFormat] AEF (NOLOCK)
				ON AEF.[Instance] = EF.[Instance]
					AND AEF.[IdEvaluationFormat] = EF.[ID]
			INNER JOIN [dbo].[Ability] A (NOLOCK)
				ON A.[Instance] = EF.[Instance]
					AND A.[ID] = AEF.[IdAbility]
			INNER JOIN [dbo].[NLU] N (NOLOCK) 
				ON N.[VCC] = EF.[Instance] 
					AND N.[Name] = SUBSTRING(A.[Query], 0, CHARINDEX('|', A.[Query]))
			WHERE EF.[Instance] = @Instance
				-- EF.[templateId] = 'qty-tmpl-c8d4ba51-be20-4364-a677-76367bd0c694'
				AND A.[QueryName] IS NOT NULL
				AND A.[Query] LIKE '%|%'
	)

	SELECT
		e.ID																	'id',
		e.Description															'description',
		e.EnvironmentType														'environmentType',
		e.Type																	'type',
		e.MinQualificationAcceptable											'minQualificationAcceptable',
		STUFF((
			SELECT ',' + CTN_INNER.[Culture]
				FROM CTE_FORMAT_NLUS CTN_INNER (NOLOCK)
				WHERE CTN_INNER.[TemplateId] = e.[templateId]
		FOR XML PATH(''), TYPE).value('.', 'NVARCHAR(MAX)'), 1, 1, '')			'modelLanguages',
		e.CreatedDate															'createdDate',
		e.CreatedByUserId														'createdByUserId',
		e.LastModifiedDate														'lastModifiedDate',
		e.LastModifiedByUserId													'lastModifiedByUserId',
		e.[Enabled]																'enabled',
		e.[templateId]															'templateId'
	FROM EvaluationFormat e (NOLOCK)
	WHERE e.Instance = @Instance
		AND ISNULL(e.ID, '') LIKE CASE WHEN @SearchCriteria = 'id' THEN '%' + @SearchExpression + '%' ELSE '%' END
		AND ISNULL(e.Description, '') LIKE CASE WHEN @SearchCriteria = 'description' THEN '%' + @SearchExpression + '%' COLLATE Latin1_General_CI_AI ELSE '%' END
		AND ISNULL(@Disabled, 0) <> [Enabled]

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
