SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetEvaluations] (
	@Instance Varchar(50) = NULL,
	@IdAuditor VARCHAR(MAX) = NULL,
	@IdAgent Varchar(MAX) = NULL,
   	@IdCampaign VARCHAR(MAX) = NULL,
	@IdEvaluationPlan VARCHAR(MAX) = NULL,
 	@EvaluationDateFrom VARCHAR(50) = '1980-01-01T00:00:00',
    @EvaluationDateTo VARCHAR(50) = '2050-01-01T23:59:59',
	@EvaluationHourFrom VARCHAR(50) = '00:00:00',
    @EvaluationHourTo VARCHAR(50) = '23:59:59',
	@InteractionMinDuration Int = 0,
    @InteractionMaxDuration Int = 86400,
    @InteractionOrigins Varchar(50) = NULL,
	@InteractionResults Varchar(MAX) = NULL,
	@EvaluationTypes Varchar(MAX) = NULL,
	@InteractionTags Varchar(MAX) = NULL,
	@InteractionFormats Varchar(50) = NULL,
	@InteractionCoached VARCHAR(50) = NULL,
	@InteractionIds     VARCHAR(MAX) = NULL,
	@EvaluationsLimit Int = 0,
	@ContactAddress Varchar(300) = NULL,
	@SentimentIds VARCHAR(MAX) = NULL
)
AS
BEGIN
	SET NOCOUNT ON
	 -- SENTIMIENTOS
	 DECLARE @FILTERS_SENTIMENTIDS VARCHAR(MAX)
	 DECLARE @FILTER_ANALYSIS_TYPE VARCHAR(MAX)
	 SET @FILTERS_SENTIMENTIDS = ''
	 SET @FILTER_ANALYSIS_TYPE = ''
	IF (@SentimentIds IS NOT NULL AND @SentimentIds <> '')
    BEGIN		
		SET @FILTERS_SENTIMENTIDS += '  AND SAR.[SentimentId] IN (SELECT Value FROM [dbo].[split](''' + @SentimentIds + ''','','')) '										
	END
	 
	 IF (SELECT COUNT(*) FROM [dbo].[split](@InteractionOrigins, ',')) = 2 
		SET  @FILTER_ANALYSIS_TYPE =  ' 
				AND SAR.[AnalysisType] ='+''''+'general'+''''
	 IF (@InteractionOrigins = 'IN') 
		SET  @FILTER_ANALYSIS_TYPE =  ' 
				AND SAR.[AnalysisType] ='+''''+'client'+''''
	 IF (@InteractionOrigins = 'OUT') 
		SET  @FILTER_ANALYSIS_TYPE =  ' 
				AND SAR.[AnalysisType] ='+''''+'agent'+''''
	 -- FIN SENTIMIENTOS 
	
	IF (@EvaluationHourFrom = '') SET @EvaluationHourFrom = '00:00:00';
	IF (@EvaluationHourTo = '') SET @EvaluationHourTo = '23:59:59';
	
	-- SI ORIGINS TIENE DOS VALORES, IGNORAMOS EL FILTRO
	IF (SELECT COUNT(*) FROM [dbo].[split](@InteractionOrigins, ',')) = 2 SET @InteractionOrigins = ''
	
	DECLARE @SQL_TMP VARCHAR(MAX) = ''
	DECLARE @SQL_CTE VARCHAR(MAX) = ''
	DECLARE @SQL_CTE_SELECT VARCHAR(MAX) = ''
	DECLARE @SQL_CTE_FROM_FIXED VARCHAR(MAX) = ''
	DECLARE @SQL_CTE_FROM_OPTIONAL VARCHAR(MAX) = ''
	DECLARE @SQL_CTE_FILTERS_FIXED VARCHAR(MAX) = ''
	DECLARE @SQL_CTE_FILTERS_OPTIONAL VARCHAR(MAX) = ''
	DECLARE @SQL_CTE_FILTERS_EXCLUDING VARCHAR(MAX) = ''

	DECLARE @SQL_SELECT_1 VARCHAR(MAX) = ''
	DECLARE @SQL_FROM VARCHAR(MAX) = ''
	DECLARE @SQL_FILTER VARCHAR(MAX) = ''
	DECLARE @SQL_EXTRA VARCHAR(MAX) = ''
	DECLARE @FILTERS_OPTIONAL_PART VARCHAR(MAX) = ''
    DECLARE @FILTERS_EXCLUDING_PART VARCHAR(MAX) = ''
    DECLARE @FILTERS_FIXED_PART     VARCHAR(MAX) = ''

	-- INICIAMOS ARMANDO LOS FILTROS PARA LA TABLA CTE PRINCIPAL
	SET @SQL_CTE_FILTERS_FIXED = '
			WHERE ER.[Instance] = ''' + @Instance + '''
				AND ER.[EvaluationDate] BETWEEN ''' + @EvaluationDateFrom + '''AND ''' + @EvaluationDateTo + '''
				AND EP.[Status] = CASE EP.[Evaluation_Mode] WHEN ''A'' THEN 7 ELSE EP.[Status] END 
				AND AEP.[Completed] = 1 '

	-- AGREGAMOS EL SCRIPT PARA LOS FILREOS DE HORA MAXIMA Y HORA MINIMA LAS CUALES TIENEN UN COMPORTAMIENTO DIFERENTES SEGUN CUAL ES MAYOR
	IF (CAST(@EvaluationHourFrom AS TIME) <= CAST(@EvaluationHourTo AS TIME)) 
		SET @SQL_CTE_FILTERS_OPTIONAL += '
				AND CAST(ER.[EvaluationDate] AS TIME) BETWEEN ''' + @EvaluationHourFrom + ''' AND ''' + @EvaluationHourTo + ''' '
	IF (CAST(@EvaluationHourFrom AS TIME) > CAST(@EvaluationHourTo AS TIME)) 
		SET @SQL_CTE_FILTERS_OPTIONAL += '
				AND CAST(ER.[EvaluationDate] AS TIME) NOT BETWEEN ''' + @EvaluationHourTo + ''' AND ''' + @EvaluationHourFrom + ''' '

	-- AGREGAMOS EL SCRIPT PARA LOS FILREOS DE DURACION MAXIMA Y DURACION MINIMA 
	IF (@InteractionMinDuration IS NOT NULL) 
		SET @SQL_CTE_FILTERS_OPTIONAL += '
				AND AEP.[InteractionDuration] >= ' + (SELECT CONVERT(varchar(10), @InteractionMinDuration)) + ' '
	IF (@InteractionMaxDuration IS NOT NULL) 
		SET @SQL_CTE_FILTERS_OPTIONAL += '
				AND AEP.[InteractionDuration] <= ' + (SELECT CONVERT(varchar(10), @InteractionMaxDuration))+ ' '
	
	-- AGREGAMOS EL SCRIPT PARA EL FILTRO DE ORIGIN
	IF (@InteractionOrigins IS NOT NULL AND @InteractionOrigins != '') 
		SET @SQL_CTE_FILTERS_OPTIONAL += '
				AND AEP.[InteractionOrigin] IN (SELECT Value FROM [dbo].[split](''' + @InteractionOrigins + ''','','')) '

	-- AGREGAMOS EL SCRIPT PARA EL FILTRO DE AUDITOR
	IF (@IdAuditor IS NOT NULL AND @IdAuditor <> '') 
	BEGIN
		-- DEBEMOS PASAR LA LISTA DE AUDITORS A UNA TABLA TEMPORAL PARA AGILIZAR LA BUSQUEDA 
		SELECT * 
			INTO #FILTER_AUDITORS
			FROM (
				SELECT Value FROM [dbo].[split](@IdAuditor, ',')
			) A

		-- AGREGAMOS EL JOIN EN LA BUSQUEDA PRINCIPAL
		SET @SQL_CTE_FROM_OPTIONAL += '  
			INNER JOIN #FILTER_AUDITORS FA 
				ON FA.[Value] = ER.[IdAuditor] ' 
	END

	-- AGREGAMOS EL SCRIPT PARA EL FILTRO DE AGENT
	IF (@IdAgent IS NOT NULL AND @IdAgent != '') 
	BEGIN
		-- DEBEMOS PASAR LA LISTA DE AUDITORS A UNA TABLA TEMPORAL PARA AGILIZAR LA BUSQUEDA 
		SELECT * 
			INTO #FILTER_AGENTS
			FROM (
				SELECT Value FROM [dbo].[split](@IdAgent, ',')
			) A

		-- AGREGAMOS EL JOIN EN LA BUSQUEDA PRINCIPAL
		SET @SQL_CTE_FROM_OPTIONAL += '  
			INNER JOIN #FILTER_AGENTS FAG 
				ON FAG.[Value] = ER.[IdAgent] ' 
	END

	-- AGREGAMOS EL SCRIPT PARA EL FILTRO DE CAMPAIGN
	IF (@IdCampaign IS NOT NULL AND @IdCampaign != '') 
	BEGIN
		-- DEBEMOS PASAR LA LISTA DE AUDITORS A UNA TABLA TEMPORAL PARA AGILIZAR LA BUSQUEDA 
		SELECT * 
			INTO #FILTER_CAMPAIGNS
			FROM (
				SELECT Value FROM [dbo].[split](@IdCampaign, ',')
			) A
		
		-- AGREGAMOS EL JOIN EN LA BUSQUEDA PRINCIPAL
		SET @SQL_CTE_FROM_OPTIONAL += '  
			INNER JOIN #FILTER_CAMPAIGNS FC 
				ON FC.[Value] = ER.[IdCampaign] ' 
	END

	-- AGREGAMOS EL SCRIPT PARA EL FILTRO DE EVALUATION PLAN
	IF (@IdEvaluationPlan IS NOT NULL AND @IdEvaluationPlan != '') 
	BEGIN
		-- DEBEMOS PASAR LA LISTA DE AUDITORS A UNA TABLA TEMPORAL PARA AGILIZAR LA BUSQUEDA 
		SELECT * 
			INTO #FILTER_EVALUATION_PLANS
			FROM (
				SELECT Value FROM [dbo].[split](@IdEvaluationPlan, ',')
			) A 
		
		-- AGREGAMOS EL JOIN EN LA BUSQUEDA PRINCIPAL
		SET @SQL_CTE_FROM_OPTIONAL += '  
			INNER JOIN #FILTER_EVALUATION_PLANS FEP 
				ON FEP.[Value] = ER.[IdEvaluationPlan] ' 
	END

	-- AGREGAMOS EL SCRIPT PARA EL FILTRO DE EVALUATION PLAN
	IF @ContactAddress IS NOT NULL AND @ContactAddress <> ''
	BEGIN
		SET @SQL_CTE_FROM_OPTIONAL += '
				AND AEP.[ContactAddress] LIKE ''%' + @ContactAddress + '%'' '
	END	

	-- AGREGAMOS EL SCRIPT PARA INTERACTION_ID
	IF @InteractionIds IS NOT NULL AND @InteractionIds <> ''
	BEGIN
		SET @SQL_CTE_FROM_OPTIONAL = ''
		SET @SQL_CTE_FILTERS_OPTIONAL = ''
			
		-- DEBEMOS PASAR LA LISTA DE CAMPAIGNS A UNA TABLA TEMPORAL PARA AGILIZAR LA BUSQUEDA 
		SELECT * 
			INTO #FILTER_INTERACTIONS_IDS
			FROM (
				SELECT Value FROM [dbo].[split](@InteractionIds, ',')
			) A

		-- AGREGAMOS EL JOIN EN LA BUSQUEDA PRINCIPAL
		SET @SQL_CTE_FROM_OPTIONAL += '  
			INNER JOIN #FILTER_INTERACTIONS_IDS FID 
				ON FID.[Value] = AEP.[IdInteraction] ' 
		-- SET @SQL_CTE_FILTERS_EXCLUDING = ' AND ID.[Id] IN (SELECT Value FROM [dbo].[split](''' + @InteractionIds + ''','','')) '
	END 

	-- ARMAMOS EL QUERY PARA EL CONTENIDO DE LA CTE
	SET @SQL_CTE_SELECT = '
		;WITH CTE_RESULTS AS (
			SELECT ER.[Instance],
					ER.[IdEvaluationPlan],
					ER.[IdInteraction], 
					IEP.[InteractionSection],
					IEP.[InteractionPart],
					IEP.[InteractionDateEnd],
					ER.[IdAgent],
					IEP.[AgentName],
					IEP.[CreatedDate]
					-- SAR.[SentimentId]
				FROM [dbo].[EvaluationResult] ER (NOLOCK) 
				INNER JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
					ON EP.[Instance] = ER.[Instance]
						AND EP.[Id] =  ER.[IdEvaluationPlan]
				INNER JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK) 
					ON (AEP.[Instance] = ER.[Instance] 
						AND AEP.[IdAuditor] = ER.[IdAuditor] 
						AND AEP.[IdInteraction] = ER.[IdInteraction] 
						AND AEP.[InteractionType] = ER.[InteractionType] 
						AND AEP.[IdEvaluationPlan] = ER.[IdEvaluationPlan]) 
				INNER JOIN [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
					ON IEP.[Instance] = ER.[Instance] 
						AND IEP.[IdEvaluationPlan] = EP.[ID]
						AND IEP.[IdInteraction] = ER.[IdInteraction]
						AND IEP.[IdAgent] = ER.[IdAgent] '
					+ @SQL_CTE_FROM_OPTIONAL
					+ @SQL_CTE_FILTERS_FIXED
					+ @SQL_CTE_FILTERS_OPTIONAL
					+ @SQL_CTE_FILTERS_EXCLUDING + '
		)
		,CTE_WITH_SAR AS ( 
			SELECT CTE_R.*,
					SAR.[SentimentId],
					RN = ROW_NUMBER() OVER(PARTITION BY CTE_R.[Instance], CTE_R.[IdEvaluationPlan], CTE_R.[IdInteraction] ORDER BY CTE_R.[CreatedDate] ASC) 
				FROM CTE_RESULTS CTE_R
				LEFT JOIN [dbo].[SentimentAnalysisResult] SAR (NOLOCK)
					ON CTE_R.[Instance] = SAR.[VCC]
						AND CTE_R.[IdInteraction] = SAR.[InteractionId]
						AND CTE_R.[InteractionSection] = SAR.[InteractionSection]
						AND CTE_R.[InteractionPart] = SAR.[InteractionPart]
                        AND ISNULL(CTE_R.[InteractionDateEnd], 0) = ISNULL(SAR.[EndDate], 0)
                        AND SAR.[MaxScore] = 1
						' + @FILTERS_SENTIMENTIDS + '
						' + @FILTER_ANALYSIS_TYPE + '
		)
		,SUMMARIZED_CTE AS (
			SELECT CTE_S.[Instance],
					CTE_S.[IdEvaluationPlan],
					CTE_S.[IdInteraction], 
					CTE_S.[IdAgent],
					CTE_S.[AgentName],
					CTE_S.[SentimentId],
					IIF (
						ERV.[InteractionId] IS NULL,
						CAST(0 AS BIT),
						CAST(1 AS BIT)
					)										''Review'',
					COUNT(ERC.[Id])							''Commitments''
				FROM CTE_WITH_SAR CTE_S
				LEFT JOIN [dbo].[EvaluationReview] ERV (NOLOCK)
					ON ERV.[VCC] = CTE_S.[Instance]
						AND ERV.[EvaluationPlanId] = CTE_S.[IdEvaluationPlan]
						AND ERV.[InteractionId] = CTE_S.[IdInteraction]
				LEFT JOIN [dbo].[EvaluationReviewCommitment] ERC (NOLOCK)
					ON ERC.[VCC] = CTE_S.[Instance] 
						AND ERC.[EvaluationPlanId] = CTE_S.[IdEvaluationPlan]
						AND ERC.[InteractionId] = CTE_S.[IdInteraction]
						-- AND ERC.[SupervisorId] = AEP.[IdAuditor] 
						AND ERC.[AgentId] = CTE_S.[idAgent]
				WHERE RN = 1
				GROUP BY CTE_S.[Instance],
							CTE_S.[IdEvaluationPlan],
							CTE_S.[IdInteraction], 
							ERV.[InteractionId],
							CTE_S.[IdAgent],
							CTE_S.[AgentName],
							CTE_S.[SentimentId]'

	IF @InteractionCoached IS NOT NULL AND @InteractionCoached != ''
	BEGIN
		IF @InteractionCoached = 'true'
			SET @SQL_CTE_SELECT += '
				HAVING COUNT(ERC.[Id]) >= 1 '
		ELSE 
			SET @SQL_CTE_SELECT += '
				HAVING COUNT(ERC.[Id]) = 0 '
	END

	SET @SQL_CTE_SELECT += '
		)'


	SET @SQL_CTE_SELECT += '
		-- SELECT * FROM CTE'

	-- ARMAMOS EL QUERY PRINCIPAL
	SET @SQL_CTE = @SQL_TMP + @SQL_CTE_SELECT

	-- EMPEZAMOS A ARMAN LA QUERY PRINCIPAL
	SET @SQL_SELECT_1 = @SQL_CTE

	SET @SQL_SELECT_1 += '
		SELECT AEP.Completed								''completed'',
				AEP.[StatusDescription]						''statusDescription'',
				A.[ErrorMessage]							''errorMessage'',
    			AEP.[Score]									''qualification'',
    			AEP.[IdAuditor]								''idAuditor'',
    			U.[FirstName] + '' '' + U.[LastName]		''auditorName'',
    			MAX(ER.[EvaluationDate])					''evaluationDate'',
    			AEP.[IdEvaluationPlan]						''idEvaluationPlan'',
				EP.[Description]							''evaluationPlan'',
				EP.[Evaluation_Mode]						''evaluationType'',
    			AEP.[idAgent]								''idAgent'',
    			S_CTE.[AgentName]							''agentName'',							-- NOMBRE DEL AGENTE
				S_CTE.[IdInteraction]						''idInteraction'',
				AEP.[InteractionDate]						''interactionDate'',
    			AEP.[InteractionType]						''interactionType'',
    			AEP.[IdCampaign]							''idCampaign'',
				E.[Type]									''environmentType'',
				S_CTE.[Review]								''review'',
				S_CTE.[Commitments]							''commitments'',
				S_CTE.[SentimentId]							''sentimentResult'' '

	SET @SQL_FROM += '
			FROM SUMMARIZED_CTE S_CTE (NOLOCK) 
			INNER JOIN dbo.[EvaluationResult] ER (NOLOCK) 
				ON ER.[Instance] = S_CTE.[Instance]
					AND ER.[IdEvaluationPlan] = S_CTE.[IdEvaluationPlan]
					AND ER.[IdInteraction] = S_CTE.[IdInteraction]
					AND ER.[IdAgent] = S_CTE.[IdAgent]
			INNER JOIN dbo.[EvaluationPlan] EP (NOLOCK) 
				ON EP.[Instance] = S_CTE.[Instance] 
					AND EP.[ID] = S_CTE.[IdEvaluationPlan]
			INNER JOIN dbo.[AuditorEvaluationPlan] AEP (NOLOCK) 
				ON (AEP.[Instance] = ER.[Instance] 
					AND AEP.[IdAuditor] = ER.[IdAuditor] 
					AND AEP.[IdInteraction] = ER.[IdInteraction] 
					AND AEP.[InteractionType] = ER.[InteractionType] 
					AND AEP.[IdEvaluationPlan] = ER.[IdEvaluationPlan]) 
			LEFT JOIN [dbo].[Audio] A (NOLOCK)
			    ON A.[VCC] = AEP.[Instance]
			        AND A.[ID] = AEP.[IdInteraction]			
			INNER JOIN [dbo].[Environment] E (NOLOCK)
				ON E.[VCC] = ER.[Instance]
					AND E.[Name] = EP.[EnvironmentId]
			INNER JOIN dbo.[User] U (NOLOCK) 
				ON U.[VCC] = ER.[Instance] 
					AND U.[ID] = AEP.[IdAuditor] '

	-- AGREGAMOS EL SCRIPT PARA DISPOSITION CODES
	IF (@InteractionResults IS NOT NULL AND @InteractionResults != '') 
		SET @FILTERS_OPTIONAL_PART += '
				AND (
					(CONCAT(AEP.[IdCampaign], ISNULL(REPLACE(AEP.[ManagementResult], ''/'', ''_''), ''_'')) IN (SELECT Value FROM [dbo].[split](''' + @InteractionResults + ''','','')))
					OR
					(CONCAT(AEP.[IdCampaign], ''_'', ISNULL(AEP.[ManagementResult], ''_'')) IN (SELECT Value FROM [dbo].[split](''' + @InteractionResults + ''','','')))
					OR
					(AEP.[ManagementResult] IN (SELECT Value FROM [dbo].[split](''' + @InteractionResults + ''','','')))
				)'

	-- AGREGAMOS EL SCRIPT PARA EVALUATION TYPE
	IF (@EvaluationTypes IS NOT NULL AND @EvaluationTypes != '') 
	BEGIN
		SET @FILTERS_OPTIONAL_PART += '
				AND EP.[Evaluation_Mode] IN (SELECT Value FROM [dbo].[split](''' + @EvaluationTypes + ''','',''))  '
	END

	-- AGREGAMOS EL SCRIPT PARA SENTIMENT
	IF @SentimentIds IS NOT NULL AND @SentimentIds <> ''
    BEGIN            
	    SET @FILTERS_OPTIONAL_PART=' 
				AND S_CTE.[SentimentId] IN (SELECT Value FROM [dbo].[split](''' + @SentimentIds + ''','','')) '
    END

	-- AGREGAMOS EL SCRIPT PARA TAG
	IF (@InteractionTags IS NOT NULL AND @InteractionTags != '') 
	BEGIN
		SET @SQL_FROM += '
			LEFT JOIN [Tag] T (NOLOCK) 
				ON (T.[VCC] = ER.[Instance] 
					AND T.[Tag] IN (SELECT Value FROM [dbo].[split](''' + @InteractionTags + ''','',''))) '
		SET @FILTERS_OPTIONAL_PART += '
				AND T.[AudioId] LIKE CONCAT(AEP.[IdInteraction], ''%'') 
				AND T.[AudioId] IS NOT NULL '
	END

	SET @SQL_EXTRA += '
			GROUP BY AEP.[Completed],
						AEP.[StatusDescription],
						A.[ErrorMessage],
						AEP.[score],
						AEP.[IdAuditor],
						U.[FirstName] + '' '' + U.[LastName],
						AEP.[IdEvaluationPlan],
						EP.[Description],
						EP.[Evaluation_Mode],
						AEP.[idAgent],
						S_CTE.[AgentName],
						AEP.[InteractionDate],
						AEP.[InteractionType],
						AEP.[IdCampaign], 
						E.[Type],
						S_CTE.[IdInteraction],
						S_CTE.[Review],
						S_CTE.[Commitments],
						S_CTE.[SentimentId]'

	SET @SQL_EXTRA += '
			ORDER BY MAX(ER.[EvaluationDate]) DESC '

	SET @SQL_SELECT_1 += @SQL_FROM  + @FILTERS_FIXED_PART + @FILTERS_OPTIONAL_PART + @FILTERS_EXCLUDING_PART + @SQL_EXTRA 


	-- ELIMINAMOS LAS TABLAS TEMPORALES
	SET @SQL_SELECT_1 += '
			DROP TABLE IF EXISTS #FILTER_AUDITORS
			DROP TABLE IF EXISTS #FILTER_AGENTS
			DROP TABLE IF EXISTS #FILTER_CAMPAIGNS 
			DROP TABLE IF EXISTS #FILTER_EVALUATION_PLANS
			DROP TABLE IF EXISTS #FILTER_INTERACTIONS_IDS '

	-- EJECUTAMOS EL QUERY ARMADO
	-- PRINT @SQL_SELECT_1
	EXEC (@SQL_SELECT_1)
	
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
