SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetGlobalSearchInteractions] (
	@VCC Varchar(50) = NULL,
	@Agents Varchar(MAX) = NULL,
   	@Campaigns VARCHAR(MAX) = NULL,
 	@InteractionDateFrom VARCHAR(50) = '1980-01-01T00:00:00',
    @InteractionDateTo VARCHAR(50) = '2050-01-01T23:59:59',	
    @InteractionOrigins Varchar(50) = NULL,	
	@InteractionTypes Varchar(50) = NULL,
	@InteractionIds     VARCHAR(MAX) = NULL,
	@ContactAddress Varchar(300) = NULL,
	@DispositionCodes VARCHAR(MAX)
	
	
)
AS
BEGIN
	SET NOCOUNT ON	
	
	--LISTAMOS INTERACCIONES ENCONTRADAS CON LOS  FILTROS BASICOS
	
	--INTERACCIONES DE PLANES MANUALES
	--SELECT * FROM InteractionEvaluationPlan
	CREATE TABLE #AllInteractionEvaluationPlan
	(
		[interactionId]				VARCHAR(250),
		[interactionSection]		INT,
		[interactionPart]			INT,	
		[interactionType]			VARCHAR(50),
		[interactionDateStart]		DATETIME,
		[interactionDateEnd]		DATETIME,
		[agentId]					VARCHAR(50),
		[campaignId]				VARCHAR(50),
		[interactionOrigin]			VARCHAR(50),
		[contactAddress]			VARCHAR(50),
		[interactionDuration]		INT,
		[evaluationPlanId]			VARCHAR(50),
		[processId]						VARCHAR(50),
		[evaluation_Mode]			VARCHAR(50),
		RN							INT
		
	)
	IF (@InteractionIds IS NULL OR  @InteractionIds = '')
	BEGIN
		INSERT INTO #AllInteractionEvaluationPlan
			SELECT DISTINCT
			IEP.[IdInteraction]				'interactionId',
			IEP.[InteractionSection]		'interactionSection',
			IEP.[InteractionPart]			'interactionPart',	
			IEP.[InteractionType]			'interactionType',
			IEP.[InteractionDateStart]		'interactionDateStart',
			IEP.[InteractionDateEnd]		'interactionDateEnd',
			IEP.[IdAgent]					'agentId',
			IEP.[IdCampaign]				'campaignId',
			IEP.[InteractionOrigin]			'originId',
			IEP.[ContactAddress]			'contactAddress',
			IEP.[InteractionDuration]		'duration',
			IEP.[IdEvaluationPlan]			'evaluationPlanId',
			NULL							'processId',
			EP.[Evaluation_Mode]			'evaluation_mode',
			
			RN = ROW_NUMBER() OVER(PARTITION BY IEP.[Instance],IEP.[IdInteraction] ORDER BY IEP.[CreatedDate] DESC)

		
			FROM [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
				INNER JOIN dbo.[EvaluationPlan] EP (NOLOCK)
				 ON IEP.[Instance] = EP.[Instance]
					AND IEP.[IdEvaluationPlan] = EP.[ID]

			WHERE IEP.[Instance] = @VCC			
				AND	(
						ISNULL(LEN(@Agents), 0) = 0
						OR
						[IdAgent]  IN (SELECT Value FROM [dbo].[split](@Agents, ','))
					)
	
				AND (
						ISNULL(LEN(@Campaigns), 0) = 0
						OR
						[IdCampaign]  IN (SELECT Value FROM [dbo].[split](@Campaigns, ','))
					)
	
				AND (
						ISNULL(LEN(@InteractionOrigins), 0) = 0
						OR
						[InteractionOrigin]  IN (SELECT Value FROM [dbo].[split](@InteractionOrigins, ','))
					)

				AND	(
						ISNULL(LEN(@InteractionTypes), 0) = 0
						OR
						[InteractionType]  IN (SELECT Value FROM [dbo].[split](@InteractionTypes, ','))
					)	

	

				AND (
						ISNULL(LEN(@ContactAddress), 0) = 0
						OR
						[ContactAddress] LIKE '%' + @ContactAddress + '%'
					)
    
				AND (
						ISNULL(LEN(@DispositionCodes), 0) = 0
						OR
						[ManagementResult] IN (SELECT Value FROM [dbo].[split](@DispositionCodes, ','))
					)
				AND (
						IEP.InteractionDateEnd BETWEEN @InteractionDateFrom AND @InteractionDateTo
					)

					
	END
	ELSE 
	BEGIN
		INSERT INTO #AllInteractionEvaluationPlan
			SELECT DISTINCT
			IEP.[IdInteraction]				'interactionId',
			IEP.[InteractionSection]		'interactionSection',
			IEP.[InteractionPart]			'interactionPart',	
			IEP.[InteractionType]			'interactionType',
			IEP.[InteractionDateStart]		'interactionDateStart',
			IEP.[InteractionDateEnd]		'interactionDateEnd',
			IEP.[IdAgent]					'agentId',
			IEP.[IdCampaign]				'campaignId',
			IEP.[InteractionOrigin]			'originId',
			IEP.[ContactAddress]			'contactAddress',
			IEP.[InteractionDuration]		'duration',
			IEP.[IdEvaluationPlan]			'evaluationPlanId',
			NULL							'processId',
			EP.[Evaluation_Mode]			'evaluation_mode',
			
			RN = ROW_NUMBER() OVER(PARTITION BY IEP.[Instance],IEP.[IdInteraction] ORDER BY IEP.[CreatedDate] DESC)

		
			FROM [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
				INNER JOIN dbo.[EvaluationPlan] EP (NOLOCK)
				 ON IEP.[Instance] = EP.[Instance]
					AND IEP.[IdEvaluationPlan] = EP.[ID]

			WHERE IEP.[Instance] = @VCC			
					AND [IdInteraction]  IN (SELECT Value FROM [dbo].[split](@InteractionIds, ','))				
		
	END

		DROP TABLE IF EXISTS #DistinctInteractionEvaluationPlan
	    SELECT [interactionId],
				[interactionSection],
				[interactionPart],	
				[interactionType],
				[interactionDateStart],
				[interactionDateEnd],
				[agentId],
				[campaignId],
				[interactionOrigin],
				[contactAddress],
				[interactionDuration],
				[evaluationPlanId],
				[processId],
				[evaluation_Mode]
			INTO  #DistinctInteractionEvaluationPlan
			FROM #AllInteractionEvaluationPlan 
			WHERE RN = 1

	
	--INTERACCIONES DE CALL DE PLANES AUTOMATICOS QUE AUN NO EXISTAN EN EL PRIMER LISTADO

	CREATE TABLE #AllInteractionProcess
	(
		[interactionId]				VARCHAR(250),
		[interactionSection]		INT,
		[interactionPart]			INT,	
		[interactionType]			VARCHAR(50),
		[interactionDateStart]		DATETIME,
		[interactionDateEnd]		DATETIME,
		[agentId]					VARCHAR(50),
		[campaignId]				VARCHAR(50),
		[interactionOrigin]			VARCHAR(50),
		[contactAddress]			VARCHAR(50),
		[interactionDuration]		INT,
		[evaluationPlanId]			VARCHAR(50),
		[processId]						VARCHAR(50),
		[evaluation_Mode]			VARCHAR(50),
		RN							INT
		
	)

	IF (@InteractionIds IS NULL OR  @InteractionIds = '')
	BEGIN
		INSERT INTO #AllInteractionProcess
			SELECT DISTINCT
			A.[ID]					'interactionId',
			A.[Section]				'interactionSection',
			A.[Part]				'interactionPart',	
			A.[Channel]				'interactionType',
			A.[StartDate]			'interactionDateStart',
			A.[EndDate]				'interactionDateEnd',
			A.[Agent]				'agentId',
			A.[Campaign]			'campaignId',
			A.[Type]				'originId',
			A.[ContactAddress]		'contactAddress',
			A.[DurationTime]		'duration',
			NULL					'evaluationPlanId',
			PA.[ProcessId]			'processId',
			'A'						'evaluation_mode',
			RN = ROW_NUMBER() OVER(PARTITION BY A.[VCC],A.[ID] ORDER BY A.[CreatedDate] DESC)
	

			FROM dbo.[Audio] A (NOLOCK)
			INNER JOIN dbo.[ProcessAudio] PA (NOLOCK)
				ON  A.[VCC] = PA.[VCC]
					AND A.[ID] = PA.[AudioId] 
					AND A.[Section] = PA.[Section]
					AND A.[Part] = PA.[Part]

			WHERE A.[VCC] = @VCC 
				AND  A.[IsTranscribed] = 1
				AND	(
						ISNULL(LEN(@Agents), 0) = 0
						OR
						[Agent]  IN (SELECT Value FROM [dbo].[split](@Agents, ','))
					)
	
				AND (
						ISNULL(LEN(@Campaigns), 0) = 0
						OR
						[Campaign]  IN (SELECT Value FROM [dbo].[split](@Campaigns, ','))
					)
		
				AND (
						ISNULL(LEN(@InteractionOrigins), 0) = 0
						OR
						[Type]  IN (SELECT Value FROM [dbo].[split](@InteractionOrigins, ','))
					)

				AND	(
						ISNULL(LEN(@InteractionTypes), 0) = 0
						OR
						[channel]  IN (SELECT Value FROM [dbo].[split](@InteractionTypes, ','))
					)	

				--AND (
				--		ISNULL(LEN(@InteractionIds), 0) = 0
				--		OR
				--		[Id]  IN (SELECT Value FROM [dbo].[split](@InteractionIds, ','))
				--	)

				AND (
						ISNULL(LEN(@ContactAddress), 0) = 0
						OR
						[ContactAddress] LIKE '%' + @ContactAddress + '%'
					)
		
				AND (
						ISNULL(LEN(@DispositionCodes), 0) = 0
						OR
						[Description] IN (SELECT Value FROM [dbo].[split](@DispositionCodes, ','))
					)

				AND (
						A.EndDate BETWEEN @InteractionDateFrom AND @InteractionDateTo
					)

				AND  [Id] NOT IN (SELECT interactionId FROM #DistinctInteractionEvaluationPlan)
					
				
				
	END		
	ELSE
	BEGIN
		INSERT INTO #AllInteractionProcess
			SELECT DISTINCT
			A.[ID]					'interactionId',
			A.[Section]				'interactionSection',
			A.[Part]				'interactionPart',	
			A.[Channel]				'interactionType',
			A.[StartDate]			'interactionDateStart',
			A.[EndDate]				'interactionDateEnd',
			A.[Agent]				'agentId',
			A.[Campaign]			'campaignId',
			A.[Type]				'originId',
			A.[ContactAddress]		'contactAddress',
			A.[DurationTime]		'duration',
			NULL					'evaluationPlanId',
			PA.[ProcessId]			'processId',
			'A'						'evaluation_mode',
			RN = ROW_NUMBER() OVER(PARTITION BY A.[VCC],A.[ID] ORDER BY A.[CreatedDate] DESC)
	

			FROM dbo.[Audio] A (NOLOCK)
			INNER JOIN dbo.[ProcessAudio] PA (NOLOCK)
				ON  A.[VCC] = PA.[VCC]
					AND A.[ID] = PA.[AudioId] 
					AND A.[Section] = PA.[Section]
					AND A.[Part] = PA.[Part]

			WHERE A.[VCC] = @VCC
					AND A.[IsTranscribed] = 1
					AND A.[ID]  IN (SELECT Value FROM [dbo].[split](@InteractionIds, ','))	
					AND A.[ID] NOT IN (SELECT interactionId FROM #DistinctInteractionEvaluationPlan)
					
					
				
				
	END

	DROP TABLE IF EXISTS #DistinctInteractionProcess
	    SELECT [interactionId],
				[interactionSection],
				[interactionPart],	
				[interactionType],
				[interactionDateStart],
				[interactionDateEnd],
				[agentId],
				[campaignId],
				[interactionOrigin],
				[contactAddress],
				[interactionDuration],
				[evaluationPlanId],
				[processId],
				[evaluation_Mode]
			INTO  #DistinctInteractionProcess
			FROM #AllInteractionProcess WHERE RN = 1	

	--LISTAMOS INTERACCIONES DE TEXTO DE PLANES AUTOMATICOS QUE AUN NO EXISTAN EN UN PLAN DE EVALUACION 
	CREATE TABLE #AllTextInteractionProcess
	(
		[interactionId]				VARCHAR(250),
		[interactionSection]		INT,
		[interactionPart]			INT,	
		[interactionType]			VARCHAR(50),
		[interactionDateStart]		DATETIME,
		[interactionDateEnd]		DATETIME,
		[agentId]					VARCHAR(50),
		[campaignId]				VARCHAR(50),
		[interactionOrigin]			VARCHAR(50),
		[contactAddress]			VARCHAR(50),
		[interactionDuration]		INT,
		[evaluationPlanId]			VARCHAR(50),
		[processId]						VARCHAR(50),
		[evaluation_Mode]			VARCHAR(50),
		RN							INT
		
	)
	IF (@InteractionIds IS NULL OR  @InteractionIds = '')
	BEGIN
		INSERT INTO #AllTextInteractionProcess
			SELECT DISTINCT
			ASY.[ID]				'interactionId',
			0						'interactionSection',
			0						'interactionPart',
			ASY.[Channel]			'interactionType',
			ASY.[StartDate]			'interactionDateStart',
			ASY.[EndDate]			'interactionDateEnd',
			ASY.[FirstTakedAgent]	'agentId',
			ASY.[Campaign]			'campaignId',
			ASY.[Initiative]		'originId',
			ASY.[ContactAddress]	'contactAddress',
			ASY.[DurationTime]		'duration',
			NULL					'evaluationPlanId',
			PASY.[ProcessId]		'processId',
			'A'						'evaluation_mode',
			RN = ROW_NUMBER() OVER(PARTITION BY ASY.[VCC],ASY.[ID] ORDER BY ASY.[CreatedDate] ASC)


			FROM dbo.[AsyncInteraction] ASY (NOLOCK)
			INNER JOIN dbo.[ProcessAsyncInteraction] PASY  (NOLOCK)
				ON ASY.[VCC] = PASY.[VCC]
					AND ASY.[ID] = PASY.[InteractionId]
					AND ASY.[EndDate] = PASY.[EndDate]


			WHERE ASY.[VCC] = @VCC	
				AND ASY.[IsTranscribed] = 1
				AND	(
						ISNULL(LEN(@Agents), 0) = 0
						OR
						[FirstTakedAgent]  IN (SELECT Value FROM [dbo].[split](@Agents, ','))
					)
	
				AND	(
						ISNULL(LEN(@Campaigns), 0) = 0
						OR
						ASY.[Campaign]  IN (SELECT Value FROM [dbo].[split](@Campaigns, ','))
					)
		
				AND (
						ISNULL(LEN(@InteractionOrigins), 0) = 0
						OR
						[Initiative]  IN (SELECT Value FROM [dbo].[split](@InteractionOrigins, ','))
					)

				AND	(
						ISNULL(LEN(@InteractionTypes), 0) = 0
						OR
						[channel]  IN (SELECT Value FROM [dbo].[split](@InteractionTypes, ','))
					)	

				--AND (
				--		ISNULL(LEN(@InteractionIds), 0) = 0
				--		OR
				--		[Id]  IN (SELECT Value FROM [dbo].[split](@InteractionIds, ','))
				--	)

				AND (
						ISNULL(LEN(@ContactAddress), 0) = 0
						OR
						[ContactAddress] LIKE '%' + @ContactAddress + '%'
					)
		
				AND (
						ISNULL(LEN(@DispositionCodes), 0) = 0
						OR
						[DispositionTreePath] IN (SELECT Value FROM [dbo].[split](@DispositionCodes, ','))
					)

				AND ASY.EndDate BETWEEN @InteractionDateFrom AND @InteractionDateTo
					
				AND ASY.[ID] NOT IN (SELECT interactionId FROM #DistinctInteractionEvaluationPlan)

				
	END
	ELSE
	BEGIN
		INSERT INTO #AllTextInteractionProcess
			SELECT DISTINCT
			ASY.[ID]				'interactionId',
			0						'interactionSection',
			0						'interactionPart',
			ASY.[Channel]			'interactionType',
			ASY.[StartDate]			'interactionDateStart',
			ASY.[EndDate]			'interactionDateEnd',
			ASY.[FirstTakedAgent]	'agentId',
			ASY.[Campaign]			'campaignId',
			ASY.[Initiative]		'originId',
			ASY.[ContactAddress]	'contactAddress',
			ASY.[DurationTime]		'duration',
			NULL					'evaluationPlanId',
			PASY.[ProcessId]		'processId',
			'A'						'evaluation_mode',
			RN = ROW_NUMBER() OVER(PARTITION BY ASY.[VCC],ASY.[ID] ORDER BY ASY.[CreatedDate] ASC)


			FROM dbo.[AsyncInteraction] ASY
			INNER JOIN dbo.[ProcessAsyncInteraction] PASY 
				ON ASY.[VCC] = PASY.[VCC]
					AND ASY.[ID] = PASY.[InteractionId]
					AND ASY.[EndDate] = PASY.[EndDate]


			WHERE ASY.[VCC] = @VCC	
					AND ASY.[IsTranscribed] = 1
					AND ASY.[ID]  IN (SELECT Value FROM [dbo].[split](@InteractionIds, ','))	
					AND ASY.[ID] NOT IN (SELECT interactionId FROM #DistinctInteractionEvaluationPlan)
		

	END

	
	
	DROP TABLE IF EXISTS #DistinctTextInteractionProcess
	    SELECT [interactionId],
				[interactionSection],
				[interactionPart],	
				[interactionType],
				[interactionDateStart],
				[interactionDateEnd],
				[agentId],
				[campaignId],
				[interactionOrigin],
				[contactAddress],
				[interactionDuration],
				[evaluationPlanId],
				[processId],
				[evaluation_Mode]
			INTO   #DistinctTextInteractionProcess
			FROM #AllTextInteractionProcess WHERE RN = 1	
	
	--LISTADO CONSOLIDADO 
	
	SELECT  [interactionId],
				[interactionSection],
				[interactionPart],	
				[interactionType],
				[interactionDateStart],
				[interactionDateEnd],
				[agentId],
				[campaignId],
				[interactionOrigin],
				[contactAddress],
				[interactionDuration],
				[evaluationPlanId],
				[processId],
				[evaluation_Mode] 
	FROM #DistinctInteractionEvaluationPlan
	UNION
	SELECT  [interactionId],
				[interactionSection],
				[interactionPart],	
				[interactionType],
				[interactionDateStart],
				[interactionDateEnd],
				[agentId],
				[campaignId],
				[interactionOrigin],
				[contactAddress],
				[interactionDuration],
				[evaluationPlanId],
				[processId],
				[evaluation_Mode] 
	FROM #DistinctInteractionProcess
	UNION
	SELECT  [interactionId],
				[interactionSection],
				[interactionPart],	
				[interactionType],
				[interactionDateStart],
				[interactionDateEnd],
				[agentId],
				[campaignId],
				[interactionOrigin],
				[contactAddress],
				[interactionDuration],
				[evaluationPlanId],
				[processId],
				[evaluation_Mode] 
	FROM #DistinctTextInteractionProcess
		
	SET NOCOUNT OFF
	
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
