SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetProcessFiltersData] ( 
	@VCC VARCHAR(50) = NULL, 
	@ProcessId VARCHAR(50) = NULL
)
AS
BEGIN
	-- ELIMINAMOS LA TABLA TEMPORAL EN CASO EXISTA
	DROP TABLE IF EXISTS  #ProcessFiltersData 

	-- CREAMOS LA TABLA TEMPORAL PARA LOS RESUMENES
	CREATE TABLE #ProcessFiltersData (
		[Channel] VARCHAR(50),
		[Campaign] VARCHAR (50),
		[AgentId] VARCHAR(50),
		[AgentName] VARCHAR(100),
		[DispositionCode] VARCHAR(50),
		[Category1] VARCHAR(100),
		[Category2] VARCHAR(100),
		[Category3] VARCHAR(100),
		[Description] VARCHAR(100),
	)
	
	-- CONSULTAMOS DATA DE LA TABLA
	--SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = 'AsyncInteraction' AND COLUMN_NAME IN ('Agent', 'AgentName', 'FirstTakedAgent', 'FirstTakedAgentName', 'LastTakedAgent', 'LastTakedAgentName')
	--SELECT * FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = 'Audio' AND COLUMN_NAME IN ('Category1', 'Category2', 'Category3', 'Description', 'DispositionCode')

	-- INSERTAMOS LA DATA DE AUDIOS
	INSERT INTO #ProcessFiltersData
		SELECT 'CALL',
				A.[Campaign],
				A.[Agent],
				A.[AgentName],
				A.[DispositionCode],
				A.[Category1],
				A.[Category2],
				A.[Category3],
				A.[Description]
			FROM [dbo].[ProcessAudio] PA (NOLOCK) 
			INNER JOIN [Audio] A (NOLOCK)
				ON A.[VCC] = PA.[VCC]
					AND A.[ID] = PA.[AudioId]
					AND A.[Section] = PA.[Section]
					AND A.[Part] = PA.[Part]
			WHERE PA.[VCC] = @VCC
				AND PA.[ProcessId] = @ProcessId

	-- INSERTAMOS LA DATA DE TEXTOS
	INSERT INTO #ProcessFiltersData
		SELECT AI.[Channel],
				AI.[Campaign],
				ISNULL(AI.[LastTakedAgent], AI.[FirstTakedAgent]),
				ISNULL(AI.[LastTakedAgentName], AI.[FirstTakedAgentName]),
				AI.[DispositionCode],
				NULL,--AI.[Category1],
				NULL,--AI.[Category2],
				NULL,--AI.[Category3],
				NULL--AI.[Description]
			FROM [dbo].[ProcessAsyncInteraction] PAI (NOLOCK) 
			INNER JOIN [AsyncInteraction] AI (NOLOCK)
				ON AI.[VCC] = PAI.[VCC] 
					AND AI.[ID] = PAI.[InteractionId]
					AND AI.[Folder] = PAI.[FormattedEndDate]
					AND AI.[Campaign] = PAI.[Campaign]
			WHERE PAI.[VCC] = @VCC
				AND PAI.[ProcessId] = @ProcessId

	-- SELECCIONAMOS AGRUPADO POR FILTRO
	SELECT [Channel] 'channel' FROM #ProcessFiltersData GROUP BY [Channel] ORDER BY [Channel] ASC
	SELECT [Campaign] 'campaignId' FROM #ProcessFiltersData GROUP BY [Campaign] ORDER BY [Campaign] ASC
	SELECT [AgentId] 'agentId', 
			[AgentName] 'agentName'
		FROM #ProcessFiltersData 
		GROUP BY [AgentId], [AgentName] 
		ORDER BY [AgentId], [AgentName] ASC
	SELECT [DispositionCode]	'dispositionCode',
			[Category1]			'description',
			[Category2]			'description2',
			[Category3]			'description3',
			[Description]		'description4'
		FROM #ProcessFiltersData 
		GROUP BY [DispositionCode], [Category1], [Category2], [Category3], [Description] 
		ORDER BY [DispositionCode], [Category1], [Category2], [Category3], [Description]  ASC

	-- ELIMINAMOS LA TABLA TEMPORAL
	DROP TABLE #ProcessFiltersData 

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
