SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetProcessInteractionsHistory](
	@VCC		VARCHAR(50),
	@ProcessId	VARCHAR(50),
	@SecondsPerRequest DECIMAL(18, 5) = 1
) AS
BEGIN
	SET NOCOUNT ON

	BEGIN

		-- RECUPERAMOS LA INFORMACION DE LOS AUDIOS DEL PROCESS QUE YA HAN SIDO TRANSCRITOS 
		SELECT 'CALL'																															'interactionType',
				COUNT(*)																														'totalAudio',
				SUM(CASE WHEN A.[IsTranscribed] = 1 THEN 1 ELSE 0 END)																			'alreadyTranscribedAudio',
				SUM(CASE WHEN A.[IsCancel] = 1 THEN 1 ELSE 0 END)																				'alreadyCanceledAudio',
				SUM(CASE WHEN A.[IsTranscribed] = 1 THEN A.[DurationTime] ELSE 0 END)															'alreadyTranscribedDurationTime',
				SUM(CASE WHEN A.[IsTranscribed] = 1 THEN (CEILING(A.[DurationTime] / 60) * 60) ELSE 0 END)										'alreadyTranscribedRoundedDurationTime',
				SUM(CASE WHEN A.[IsTranscribed] = 1 THEN A.[DurationLinguisticTime] ELSE 0 END)													'alreadyTranscribedValidDurationTime',
				SUM(CASE WHEN A.[IsTranscribed] = 1 THEN (CEILING(A.[DurationLinguisticTime] / 60) * 60) ELSE 0 END)							'alreadyTranscribedRoundedValidDurationTime',
				SUM(CASE WHEN (A.[IsProcess] = 1 AND A.[IsTranscribed] IS NULL AND A.[IsCancel] IS NULL) THEN 1 ELSE 0 END)						'inProcessAudio',
				SUM(CASE WHEN (A.[IsProcess] = 1 AND A.[IsTranscribed] IS NULL AND A.[IsCancel] IS NULL) THEN A.[DurationTime] ELSE 0 END)		'inProcessDurationTime',
				SUM(CASE WHEN (A.[IsProcess] IS NULL OR (A.[IsProcess] = 1 AND A.[IsCancel] = 1)) THEN 1 ELSE 0 END)							'notTranscribedAudio',
				SUM(CASE WHEN (A.[IsProcess] IS NULL OR (A.[IsProcess] = 1 AND A.[IsCancel] = 1)) THEN A.[DurationTime] ELSE 0 END)				'notTranscribedDurationTime',
				--SUM(A.[DurationTime] / @SecondsPerRequest)																						'averageModelRequest',
				SUM(CASE WHEN A.[TotalPhrases] IS NULL THEN (A.[DurationTime] / @SecondsPerRequest) ELSE 0 END)									'averageModelRequest',
				ISNULL(SUM(A.[TotalPhrases]), 0)																								'totalCallPhrases',
				0																																'totalTextPhrases'
			FROM [dbo].[ProcessAudio] PA (NOLOCK)
			LEFT JOIN [dbo].[Audio] A (NOLOCK) 
				ON A.[VCC] =  PA.[VCC]
					AND A.[ID] = PA.[AudioId]
					AND A.[Section] = PA.[Section]
					AND A.[Part] = PA.[Part]
			WHERE PA.[VCC] = @VCC
				AND PA.[ProcessID] = @ProcessId
				-- AND A.[IsTranscribed] = 1 
				-- AND ISNULL(A.[IsProcess], 0) = 0

		SELECT PAI.[InteractionType]																											'interactionType',
				COUNT(*)																														'totalAsyncInteraction',
				SUM(CASE WHEN AI.[IsTranscribed] = 1 THEN 1 ELSE 0 END)																			'alreadyTranscribedAsyncInteraction',
				SUM(CASE WHEN AI.[IsCancel] = 1 THEN 1 ELSE 0 END)																				'alreadyCanceledAsyncInteraction',
				SUM(CASE WHEN AI.[IsTranscribed] = 1 THEN AI.[RawChar] ELSE 0 END)																'alreadyTranscribedRawChar',
				SUM(CASE WHEN AI.[IsTranscribed] = 1 THEN AI.[ValidChar] ELSE 0 END)															'alreadyTranscribedValidChar',
				SUM(CASE WHEN (AI.[IsProcess] = 1 AND AI.[IsTranscribed] IS NULL AND AI.[IsCancel] IS NULL) THEN 1 ELSE 0 END)					'inProcessAsyncInteraction',
				SUM(CASE WHEN AI.[IsProcess] = 1 THEN AI.[RawChar] ELSE 0 END)																	'inProcessRawChar',
				SUM(CASE WHEN (AI.[IsProcess] IS NULL OR (AI.[IsProcess] = 1 AND AI.[IsCancel] = 1)) THEN 1 ELSE 0 END)							'notTranscribedAsyncInteraction',
				SUM(CASE WHEN AI.[IsTranscribed] = 1 THEN AI.[RawChar] ELSE 0 END)																'notTranscribedRawChar',
				--SUM(A.[DurationTime] / @SecondsPerRequest)																					'averageModelRequest'
				0																																'averageModelRequest',
				0																																'totalCallPhrases',
				ISNULL(SUM(AI.[TotalPhrases]), 0)																								'totalTextPhrases'
			FROM [dbo].[ProcessAsyncInteraction] PAI (NOLOCK)
			LEFT JOIN [dbo].[AsyncInteraction] AI (NOLOCK)
				ON AI.[VCC] = PAI.[VCC]
					AND AI.[Channel] = PAI.[InteractionType]
					AND AI.[ID] = PAI.[InteractionId]
					AND AI.[Folder] = PAI.[FormattedEndDate]
			WHERE PAI.[ProcessId] = @ProcessId
			GROUP BY PAI.[InteractionType]
		
	END

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
