SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[GetTeamPerformanceFromEvaluationPlan] (
	@VCC					VARCHAR(50) = NULL,
	@EvaluationFormatId		VARCHAR(50) = NULL,
	@EvaluationPlanIds		VARCHAR(MAX) = NULL,
	@AgentIds				VARCHAR(MAX) = NULL,
	@AbilityIds				VARCHAR(MAX) = NULL,
	@StartDate				DATETIME,
	@EndDate				DATETIME,
	@drilldownTarget		VARCHAR(50) = NULL,
	@drilldownId			VARCHAR(50) = NULL
)
AS
BEGIN

	DROP TABLE IF EXISTS #SEARCHED_INTERACTIONS
	CREATE TABLE #SEARCHED_INTERACTIONS
	(
		AgentId				VARCHAR(100),
		EvaluationPlanId	VARCHAR(50),
		InteractionId		VARCHAR(250)
	)
	
	IF(@drilldownTarget ='ABILITY')
	BEGIN
		SET @AbilityIds = @drilldownId
	
		INSERT INTO #SEARCHED_INTERACTIONS
			SELECT DISTINCT ER.[IdAgent]															'AgentId',
							ER.[IdEvaluationPlan]													'EvaluationPlanId',
							ER.[IdInteraction]														'InteractionId' 
		
				FROM [dbo].[EvaluationResult] ER (NOLOCK)				
				INNER JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
					ON ER.[Instance] = EP.[Instance]
						AND ER.[IdEvaluationPlan] = EP.[ID]
				INNER JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
					ON EP.[Instance] = AEP.[Instance]
						AND EP.[ID] = AEP.[IdEvaluationPlan]
						AND (
							(
								EP.[Status] = 7 
								AND 
								EP.Evaluation_Mode = 'A'
							)
							OR
							(	
								AEP.[Completed] = 1 
								AND 
								EP.Evaluation_Mode IN ('X','M')
							)
						)
						AND ER.[IdEvaluationPlan] = AEP.[IdEvaluationPlan]
						AND ER.[IdInteraction] = AEP.[IdInteraction]
						AND ER.[IdAgent] = AEP.[IdAgent]
						AND ER.[IdAuditor] = AEP.[IdAuditor]
						AND AEP.[Completed] = 1
						AND AEP.[StatusDescription] NOT IN('INVALID') 
				WHERE ER.[Instance] = @VCC
					AND ER.[InteractionDatefin] BETWEEN @StartDate AND @EndDate
					AND ER.[IdEvaluationFormat] = @EvaluationFormatId
					AND (
						ISNULL(LEN(@EvaluationPlanIds), 0) = 0
						OR					
						ER.[IdEvaluationPlan]  IN (SELECT Value FROM [dbo].[split](@EvaluationPlanIds, ','))
					)	
					AND (
						ISNULL(LEN(@AgentIds), 0) = 0
						OR
						ER.[IdAgent]  IN (SELECT Value FROM [dbo].[split](@AgentIds, ','))
					)
					AND (
						ISNULL(LEN(@AbilityIds), 0) = 0
						OR 					
						(
							ER.[IdAbility] IN (SELECT Value FROM [dbo].[split](@AbilityIds, ','))
							AND
							(
								/* EXCEPCION PARA HABILIDADES CUALITATIVAS Y CUANTITATIVAS, LAS CUALITATIVAS TIENEN [MaxAbilityScore] = 0 */
								/* Y NO PUEDE DISTINGUIRSE CUAL ES EL SCORE SELECCIONADO, POR ESO COMPARAMOS CON [AbilityInnerScore]      */
								(
									ER.[MaxAbilityScore] <> 0
									AND
									ER.[Qualification] = ER.[MaxAbilityScore]
								)
								OR
								(
									ER.[MaxAbilityScore] = 0
									AND
									ER.[AbilityInnerScore] = 100
								)
							)
						)
					)

	END
	ELSE
	BEGIN
		INSERT INTO #SEARCHED_INTERACTIONS
			SELECT DISTINCT ER.[IdAgent]															'AgentId',
							ER.[IdEvaluationPlan]													'EvaluationPlanId',
							ER.[IdInteraction]														'InteractionId' 
				FROM [dbo].[EvaluationResult] ER (NOLOCK)
				INNER JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
					ON ER.[Instance] = EP.[Instance]
						AND ER.[IdEvaluationPlan] = EP.[ID]
				INNER JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
					ON EP.[Instance] = AEP.[Instance]
						AND EP.[ID] = AEP.[IdEvaluationPlan]
						AND (
							(
								EP.[Status] = 7 
								AND 
								EP.Evaluation_Mode = 'A'
							)
							OR
							(
								AEP.[Completed] = 1 
								AND 
								EP.Evaluation_Mode IN ('X','M')
							)
						)
						AND ER.[IdEvaluationPlan] = AEP.[IdEvaluationPlan]
						AND ER.[IdInteraction] = AEP.[IdInteraction]
						AND ER.[IdAgent] = AEP.[IdAgent]
						AND ER.[IdAuditor] = AEP.[IdAuditor]
						AND AEP.[Completed] = 1
						AND AEP.[StatusDescription] NOT IN('INVALID') 

				WHERE ER.[Instance] = @VCC
					AND ER.[InteractionDatefin] BETWEEN @StartDate AND @EndDate
					AND ER.[IdEvaluationFormat] = @EvaluationFormatId
					AND (
						ISNULL(LEN(@EvaluationPlanIds), 0) = 0
						OR					
						ER.[IdEvaluationPlan]  IN(SELECT Value FROM [dbo].[split](@EvaluationPlanIds,','))
						)
					
					AND (
						ISNULL(LEN(@AgentIds), 0) = 0
						OR
						ER.[IdAgent]  IN(SELECT Value FROM [dbo].[split](@AgentIds,','))
						)
			
					AND (
						ISNULL(LEN(@AbilityIds), 0) = 0
						OR 
						(
						ER.[IdAbility] IN (SELECT Value FROM [dbo].[split](@AbilityIds, ','))
					
						)
					)

	END


   
	-- GENERAMOS EL LISTADO DE INTERACCIONES A BUSCAR
	
	
	--SELECT * FROM #SEARCHED_INTERACTIONS
	-- OBTENEMOS TOTAL DE INTERACCIONES POR AGENTE	
	DROP TABLE IF EXISTS #SEARCHED_INTERACTIONS_TOTALBYAGENT
	SELECT [AgentId],COUNT([InteractionId]) 'Total' 
		INTO #SEARCHED_INTERACTIONS_TOTALBYAGENT
		FROM #SEARCHED_INTERACTIONS 
		GROUP BY [AgentId]
	
	--SELECT * FROM #SEARCHED_INTERACTIONS_TOTALBYAGENT
	
	--OBTENENEMOS SUMATORIA DE NOTAS POR AGENTE
	--PARA EL CASO DE MIXTAS LOS RESULTADOS SE MULTIPLICAN SEGUN EL NUMERO DE AUDITORES ES POR ELL QUE SE DEBE TOMAR 
	
	DROP TABLE IF EXISTS #SEARCHED_INTERACTIONS_NOTAS
	SELECT ER.[IdAgent]					'AgentId',
			ER.[IdEvaluationPlan]		'EvaluationPlanId',	
			ER.[IdInteraction]			'InteractionId', 
			SUM(Qualification)			'NotaTotal'
		INTO  #SEARCHED_INTERACTIONS_NOTAS
		FROM [dbo].[EvaluationResult] ER (NOLOCK)
		INNER JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
			ON ER.[IdEvaluationPlan] = AEP.[IdEvaluationPlan]
				AND ER.[IdInteraction] = AEP.[IdInteraction]
				AND ER.[IdAgent] = AEP.[IdAgent]
				AND ER.[IdAuditor] = AEP.[IdAuditor]
				AND AEP.[Completed] = 1
				AND AEP.[StatusDescription] NOT IN('INVALID')
		INNER JOIN #SEARCHED_INTERACTIONS SI
			ON ER.[IdAgent] =SI.[AgentId]
				AND ER.[IdEvaluationPlan] = SI.[EvaluationPlanId]
				AND  ER.[IdInteraction] = SI.[InteractionId]
		WHERE ER.[Instance] = @VCC
			AND ER.[IdEvaluationFormat] = @EvaluationFormatId
			AND (
				ISNULL(LEN(@EvaluationPlanIds), 0) = 0
				OR					
			    ER.[IdEvaluationPlan]  IN(SELECT Value FROM [dbo].[split](@EvaluationPlanIds,','))
			)
		GROUP BY ER.[IdAgent],
					ER.[IdEvaluationPlan],
					ER.[IdInteraction]
					
				
	--SELECT * FROM #SEARCHED_INTERACTIONS_NOTAS
	-- OBTENEMOS EL PROMEDIO DE NOTAS POR AGENTE PARA OBTENER CUARILTES
	DROP TABLE IF EXISTS #SEARCHED_INTERACTIONS_NOTAS_PROM
    SELECT [AgentId], CAST(AVG([NotaTotal]) AS DECIMAL(7,1)) 'PromNotaTotal'
	INTO #SEARCHED_INTERACTIONS_NOTAS_PROM
	FROM #SEARCHED_INTERACTIONS_NOTAS
	GROUP BY [AgentId]			

	--SELECT * FROM #SEARCHED_INTERACTIONS_NOTAS_PROM

	--OBTENEMOS CUARTILES
	--OBTENEMOS CUARTILES DE ACUERDO A LOS PROMEDIOS DE NOTAS POR AGENTE
	DROP TABLE IF EXISTS #CUARTILES
	SELECT DISTINCT CAST([PromNotaTotal] AS DECIMAL(7,1)) 'PromNotaTotal'
	INTO #CUARTILES
	FROM #SEARCHED_INTERACTIONS_NOTAS_PROM

	DROP TABLE IF EXISTS #CUARTILES_ENUM
	SELECT ROW_NUMBER() OVER(ORDER BY [PromNotaTotal] DESC) 'lux',[PromNotaTotal] 
	INTO #CUARTILES_ENUM
	FROM #CUARTILES
    
	DECLARE @CUARTIL_EL INT
	SELECT @CUARTIL_EL = COUNT(*) FROM #CUARTILES_ENUM
	
	DROP TABLE IF EXISTS #CUARTILES_FINAL
	SELECT
    CAST(CE.[PromNotaTotal] AS DECIMAL(7,1)) 'PromNotaTotal',
    IIF (A.PromNotaTotal IS NOT NULL AND @CUARTIL_EL>=4 , 1,
        IIF (B.[PromNotaTotal] IS NOT NULL AND @CUARTIL_EL>=4, 2, 
        IIF (C.[PromNotaTotal] IS NOT NULL AND @CUARTIL_EL>=4, 3, 
        IIF (D.[PromNotaTotal] IS NOT NULL AND @CUARTIL_EL>=4, 4, 1)))
    ) as quartile
	INTO #CUARTILES_FINAL
	FROM
    #CUARTILES_ENUM CE
    LEFT OUTER JOIN #CUARTILES_ENUM A on CE.PromNotaTotal = A.PromNotaTotal and A.lux > (SELECT COUNT(*)*0.00 FROM #CUARTILES_ENUM) and a.lux <= (SELECT COUNT(*)*0.25 FROM #CUARTILES_ENUM)
    LEFT OUTER JOIN #CUARTILES_ENUM B on CE.PromNotaTotal = B.PromNotaTotal and B.lux > (SELECT COUNT(*)*0.25 FROM #CUARTILES_ENUM) and b.lux <= (SELECT COUNT(*)*0.50 FROM #CUARTILES_ENUM)
    LEFT OUTER JOIN #CUARTILES_ENUM C on CE.PromNotaTotal = C.PromNotaTotal and C.lux > (SELECT COUNT(*)*0.50 FROM #CUARTILES_ENUM) and c.lux <= (SELECT COUNT(*)*0.75 FROM #CUARTILES_ENUM)
    LEFT OUTER JOIN #CUARTILES_ENUM D on CE.PromNotaTotal = D.PromNotaTotal and D.lux > (SELECT COUNT(*)*0.75 FROM #CUARTILES_ENUM)


	-- OBTENEMOS LOS TIEMPOS DE CADA AGENTE
	DROP TABLE IF EXISTS #AGENT_AUDIOS_TIME
	SELECT A.[Agent]								'AgentId',
			A.[AgentName]							'AgentName',
			AVG(A.[DurationLinguisticTime])			'AverageLinguisticTime',
			SUM(A.[DurationLinguisticTime])			'TotalLinguisticTime'
		INTO #AGENT_AUDIOS_TIME
		FROM [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
		INNER JOIN #SEARCHED_INTERACTIONS SI
			ON SI.[InteractionId] = IEP.[IdInteraction]
			AND SI.[EvaluationPlanId] = IEP.[IdEvaluationPlan]
		INNER JOIN [dbo].[Audio] A (NOLOCK)
			ON A.[VCC] = IEP.[Instance]
				AND A.[ID] = IEP.[IdInteraction]
				AND A.[Section] = IEP.[InteractionSection]
				AND A.[Part] = IEP.[InteractionPart]
		WHERE IEP.[Instance] = @VCC
			AND (
			 ISNULL(LEN(@EvaluationPlanIds), 0) = 0
			 OR
			 IEP.[IdEvaluationPlan]  IN (SELECT Value FROM [dbo].[split](@EvaluationPlanIds,','))

			)
			
		GROUP BY A.[Agent], 
					A.[AgentName]
	
	--SELECT * FROM  #AGENT_AUDIOS_TIME	
	
	-- OBTENEMOS LAS INTEARACCIONES QUE TUVIERON SENTIMIENTO POSITVO O NEGATIVO
	-- OBTENEMOS LAS INTEARACCIONES QUE TUVIERON SENTIMIENTO POSITVO O NEGATIVO
	SELECT DISTINCT SI.[AgentId],IEP.[IdEvaluationPlan],SR.[InteractionId] ,SR.[SentimentId]
		INTO #SEARCHED_INTERACTIONS_SENTIMENTS
		FROM [dbo].[SentimentAnalysisResult] SR (NOLOCK)
				INNER JOIN [dbo].[InteractionEvaluationPlan] IEP
					ON SR.[InteractionId] = IEP.[IdInteraction]
					AND SR.[InteractionSection] = IEP.[InteractionSection] 
					AND SR.[InteractionPart] = IEP.[InteractionPart]
					AND ISNULL(SR.[EndDate],'') = ISNULL(IEP.[InteractionDateEnd],'')
					AND IEP.[Instance] = @VCC					
					AND (
						 ISNULL(LEN(@EvaluationPlanIds), 0) = 0
						 OR					
						 IEP.[IdEvaluationPlan]  IN(SELECT Value FROM [dbo].[split](@EvaluationPlanIds,','))
						 )
					 AND SR.[AnalysisType] = 'Client'
					 AND SR.[Score] > 0
					 AND SR.[MaxScore] = 1
					 AND SR.[SentimentId] IN ('Positive','Negative') 
				
					INNER JOIN 
					#SEARCHED_INTERACTIONS SI 
					ON 	SR.[InteractionId] = SI.[InteractionId]
						AND IEP.[IdEvaluationPlan] = SI.[EvaluationPlanId]
						AND IEP.[IdAgent] = SI.[AgentId]
			 WHERE SR.VCC = @VCC
				AND SR.[AnalysisType] = 'Client'
				AND SR.[Score] > 0
				AND SR.[SentimentId] IN ('Positive','Negative')

		--SELECT * FROM #SEARCHED_INTERACTIONS_SENTIMENTS
			





	-- OBTENEMOS TOTAL DE INTERACCIONES POR AGENTE 
	DROP TABLE IF EXISTS #TOTALINTERACTION_SR_BY_AGENT
	SELECT [AgentId], COUNT([InteractionId]) 'TotalSentiments'
	INTO #TOTALINTERACTION_SR_BY_AGENT
	FROM  #SEARCHED_INTERACTIONS_SENTIMENTS
	GROUP BY [AgentId]


	

	DROP TABLE IF EXISTS #SENTIMENT_RESULT
	SELECT SIR.[AgentId]												'AgentId', 
		SIR.[SentimentId]												'SentimentId',
		COUNT(SIR.[InteractionId])										'Matches' ,
		CAST ((COUNT(SIR.[InteractionId])*100
		/
		CONVERT(DECIMAL(10,2),SRA.TotalSentiments)) AS DECIMAL(7,1))		'MatchesPercentage',
		SRA.[TotalSentiments]											'TotalSentiments',
	    SRAT.[Total]													'Total'
		
		INTO #SENTIMENT_RESULT
		FROM #SEARCHED_INTERACTIONS_SENTIMENTS SIR 
		INNER JOIN  #TOTALINTERACTION_SR_BY_AGENT SRA
		ON SIR.[AgentId] = SRA.[AgentId]
		INNER JOIN #SEARCHED_INTERACTIONS_TOTALBYAGENT SRAT
		ON SRAT.[AgentId] = SIR.[AgentId]
			
		GROUP BY SIR.[AgentId], SIR.[SentimentId], SRA.[TotalSentiments],[Total]	
		
    

	-- OBTENEMOS LAS HABILIDADES PARA NOMBRAR LAS COLUMNAS
		DECLARE @HABILITY_COLUMNS VARCHAR(MAX)
		SET @HABILITY_COLUMNS = STUFF(
				(
					/*
					SELECT DISTINCT ', ROUND(AVG(CAST(q.' + QUOTENAME(Ability) + ' AS float)), 2) As [' + Ability + ' Promedio]'
						FROM #TempEvaluationResult
					*/
					SELECT DISTINCT ',' + QUOTENAME(HA.[Id])
						FROM [dbo].[HistoricAbility] HA (NOLOCK) 
						WHERE HA.[VCC] = @VCC
							AND
                            (
                                ISNULL(LEN(@EvaluationPlanIds), 0) = 0
                                OR
                                HA.[EvaluationPlanId]  IN(SELECT Value FROM [dbo].[split](@EvaluationPlanIds,','))

                            ) 
                            
						FOR xml PATH (''), TYPE
				).value('.', 'VARCHAR(MAX)'), 
				1, 
				1, 
				''
		)

		DECLARE @HABILITY_COLUMNS_ISNULL VARCHAR(MAX)
		SET @HABILITY_COLUMNS_ISNULL = STUFF(
				(
					/*
					SELECT DISTINCT ', ROUND(AVG(CAST(q.' + QUOTENAME(Ability) + ' AS float)), 2) As [' + Ability + ' Promedio]'
						FROM #TempEvaluationResult
					*/
					SELECT DISTINCT ',' +'ISNULL('+ QUOTENAME(HA.[Id]) +',0) AS' + QUOTENAME(HA.[Id]) + ' '
						FROM [dbo].[HistoricAbility] HA (NOLOCK) 
						WHERE HA.[VCC] = @VCC
							AND
                            (
                                ISNULL(LEN(@EvaluationPlanIds), 0) = 0
                                OR
                                HA.[EvaluationPlanId]  IN(SELECT Value FROM [dbo].[split](@EvaluationPlanIds,','))

                            ) 
                            
						FOR xml PATH (''), TYPE
				).value('.', 'VARCHAR(MAX)'), 
				1, 
				1, 
				''
		)
		DECLARE @SENTIMENT_COLUMNS VARCHAR(100)
		DECLARE @ALL_COLUMNS VARCHAR(MAX)
		DECLARE @ALL_COLUMNS_ISNULL VARCHAR(MAX)
		SET @SENTIMENT_COLUMNS =',[Positive],[Negative]'
		SET @ALL_COLUMNS =  @HABILITY_COLUMNS + @SENTIMENT_COLUMNS
		SET @HABILITY_COLUMNS_ISNULL = @HABILITY_COLUMNS_ISNULL  +',ISNULL([Positive],0) AS [Positive],ISNULL([Negative],0) AS [Negative]'

		

		



	-- GENERAMOS EL RESUMEN DE RESULTADOS POR HABILIDAD Y MATCH POSITIVO
	DROP TABLE IF EXISTS #RESULTS_SUMMARY
	SELECT ER.[IdAgent]																	'AgentId',
			ER.[IdAbility]																'AbilityId',
			HA.[Title]																	'AbilityTitle',
			/*
			COUNT(IIF(ER.[Qualification] = ER.[MaxAbilityScore], 1, NULL))				'Matches',
			*/
			/* EXCEPCION PARA HABILIDADES CUALITATIVAS Y CUANTITATIVAS, LAS CUALITATIVAS TIENEN [MaxAbilityScore] = 0 */
			/* Y NO PUEDE DISTINGUIRSE CUAL ES EL SCORE SELECCIONADO, POR ESO COMPARAMOS CON [AbilityInnerScore]      */
			/*
			CASE WHEN ER.[MaxAbilityScore] <> 0
					THEN COUNT(IIF(ER.[Qualification] = ER.[MaxAbilityScore], 1, NULL))	
					ELSE COUNT(IIF(ER.[AbilityInnerScore] = 100, 1, NULL))
				END																		'Matches',
			*/
			COUNT(
				CASE WHEN ER.[MaxAbilityScore] <> 0
						THEN IIF(ER.[Qualification] = ER.[MaxAbilityScore], 1, NULL)
						ELSE IIF(ER.[AbilityInnerScore] = 100, 1, NULL)
					END
			)																			'Matches',
			/*
			CAST(COUNT(IIF(ER.[Qualification] = ER.[MaxAbilityScore], 1, NULL))*100
			/
			CONVERT(DECIMAL(4,2),SIT.Total) AS DECIMAL(7,1))							'MatchesPercentage',
			*/
			/* EXCEPCION PARA HABILIDADES CUALITATIVAS Y CUANTITATIVAS, LAS CUALITATIVAS TIENEN [MaxAbilityScore] = 0 */
			/* Y NO PUEDE DISTINGUIRSE CUAL ES EL SCORE SELECCIONADO, POR ESO COMPARAMOS CON [AbilityInnerScore]      */
			/*
			CASE WHEN ER.[MaxAbilityScore] <> 0
					THEN CAST(COUNT(IIF(ER.[Qualification] = ER.[MaxAbilityScore], 1, NULL)) * 100
							/
							CONVERT(DECIMAL(4, 2), SIT.Total) AS DECIMAL(7, 1))
					ELSE CAST(COUNT(IIF(ER.[AbilityInnerScore] = 100, 1, NULL)) * 100
							/
							CONVERT(DECIMAL(4, 2), SIT.Total) AS DECIMAL(7, 1))
				END																		'MatchesPercentage',

			*/
			CAST(
				COUNT(
					CASE WHEN ER.[MaxAbilityScore] <> 0
							THEN IIF(ER.[Qualification] = ER.[MaxAbilityScore], 1, NULL)
							ELSE IIF(ER.[AbilityInnerScore] = 100, 1, NULL)
						END
				) 
				* 
				100 
				/ 
				CONVERT(DECIMAL(10, 2), SIT.Total) AS DECIMAL(7, 1)
			)																			'MatchesPercentage',
			SIT.Total																	'Total'
			
			
		INTO #RESULTS_SUMMARY
		FROM [dbo].[EvaluationResult] ER (NOLOCK)
		INNER JOIN [dbo].[AuditorEvaluationPlan] AEP (NOLOCK)
			ON ER.[IdEvaluationPlan] = AEP.[IdEvaluationPlan]
				AND ER.[IdInteraction] = AEP.[IdInteraction]
				AND ER.[IdAgent] = AEP.[IdAgent]
				AND ER.[IdAuditor] = AEP.[IdAuditor]
				AND AEP.[Completed] = 1
				AND AEP.[StatusDescription] NOT IN('INVALID')

		INNER JOIN [dbo].[HistoricAbility] HA (NOLOCK)
			ON HA.[EvaluationPlanId] = ER.[IdEvaluationPlan]
				AND HA.[ID] = ER.[IdAbility]
		INNER JOIN #SEARCHED_INTERACTIONS SI 
			ON SI.[AgentId] = ER.[IdAgent]
				AND SI.[EvaluationPlanId] =ER.[IdEvaluationPlan]
				AND SI.[InteractionId] = ER.[IdInteraction]
       INNER JOIN #SEARCHED_INTERACTIONS_TOTALBYAGENT (NOLOCK) SIT 
		ON ER.[IdAgent] =SIT.[AgentId]			
		WHERE ER.[Instance] = @VCC
			AND (
				ISNULL(LEN(@EvaluationPlanIds), 0) = 0
				OR
				ER.[IdEvaluationPlan]  IN(SELECT Value FROM [dbo].[split](@EvaluationPlanIds,','))
			)
		GROUP BY ER.[IdAgent],
					ER.[IdAbility],
					HA.[Title],
					SIT.[Total]

	 UNION ALL
	 SELECT [AgentId],[SentimentId],[SentimentId],[Matches],[MatchesPercentage],[Total]
		    FROM #SENTIMENT_RESULT
	
	  --SELECT * FROM 	#RESULTS_SUMMARY
	 -- OBTENEMOS LAS COLUMANS DINAMICAS(HABILIDADES,SENTIMIENTOS) Y ESTATICAS(TOTAL, AGENTE) PARA EL GRID 
	-- OBTENEMOS LAS COLUMANS DINAMICAS(HABILIDADES,SENTIMIENTOS) Y ESTATICAS(TOTAL, AGENTE) PARA EL GRID 
	 SELECT 'HEADER_QUARTILE' 'headerName', 'quartile' 'field', 'quartile' 'customtype'	 
	 UNION ALL
	 SELECT 'HEADER_AGENT' 'headerName','AgentId' 'field', 'agent' 'customtype'
	 UNION ALL
	 SELECT 'HEADER_TOTAL_INTERACTIONS' 'headerName', 'TotalInteractions' 'field', 'totalInteractions' 'customtype'
	 UNION ALL
	 SELECT DISTINCT AbilityTitle 'headerName',AbilityId 'field', 
     CASE WHEN HA.IsNegative = 1 AND AC.ID IS NOT NULL  THEN 'negativeAbility' 
          WHEN HA.IsNegative = 0 AND AC.ID IS NOT NULL THEN  'positiveAbility'
          ELSE 'normalAbility' END  'customtype'
	 FROM #RESULTS_SUMMARY RS
		INNER JOIN HistoricAbility HA 
		ON RS.AbilityId = HA.ID 
        LEFT JOIN AbilityScoreCard AC 
        ON HA.ID = AC.ID
        AND AC.DegradeScoreCard = 1 
        

	 WHERE AbilityId NOT IN('Positive','Negative')
	 UNION ALL
	 SELECT 'HEADER_PROM_NOTA_TOTAL' 'headerName','PromNotaTotal' 'field', 'finalnote' 'customtype'
	 UNION ALL
	 SELECT DISTINCT AbilityId 'headerName',AbilityTitle 'field','sentiment' 'customtype' FROM #RESULTS_SUMMARY
	 WHERE AbilityId IN('Positive','Negative')
	 UNION ALL
	 SELECT 'TMO' 'headerName','TMO' 'field', 'TMO' 'customtype'



	 

	-- ARMAR DRILLDOWN PARA SETEAR UN NUEVO TOTAL A LA TABLA RESULTANTE
	-- REQUERIMOS LOS DATOS DE AGENTEID, DRILLDOWNTARGET= HABILIDAD O SENTIMIENTO, DRILLDOWNID=IDABILIDAD O IDSENTIMIENTO.
	DECLARE @QUERY VARCHAR(MAX)

      
		SET @QUERY  = '
        
		SELECT P.[AgentId],
				' + @ALL_COLUMNS + ',
				P.[Total],
				INNER_AAT.[AverageLinguisticTime],
				INNER_AAT.[TotalLinguisticTime]


			INTO #RESULT_SUMMARY_PIVOT
			FROM (
				SELECT [AgentId],
						[AbilityId],
						[MatchesPercentage],
						[Total]
					FROM #RESULTS_SUMMARY
			)
			X
			PIVOT (
				MAX([MatchesPercentage]) FOR [AbilityId] IN (' + @ALL_COLUMNS + ')
			) P
			
			INNER JOIN #AGENT_AUDIOS_TIME INNER_AAT
				ON   INNER_AAT.[AgentId] = P.[AgentId]
                
			ORDER BY P.[Total] DESC ' + '

			
			
			SELECT	RSP.[AgentId],' + @HABILITY_COLUMNS_ISNULL+ ',					
					SIP.[PromNotaTotal],
					CF.[quartile] AS quartile,
					RSP.[Total] AS TotalInteractions,
					CAST (RSP.[AverageLinguisticTime] AS DECIMAL(7,1)) AS TMO
					
				
			
			FROM  #RESULT_SUMMARY_PIVOT RSP
			INNER JOIN #SEARCHED_INTERACTIONS_NOTAS_PROM SIP
                ON RSP.[AgentId] = SIP.[AgentId]
               
				
			INNER JOIN #CUARTILES_FINAL CF ON 
				CF.PromNotaTotal = SIP.PromNotaTotal

			ORDER BY SIP.[PromNotaTotal] DESC
			
	    '


	-- EJECUTAMOS EL QUERY ARMADO

	
	--PRINT (@QUERY)
	EXECUTE (@QUERY)

	

	DROP TABLE IF EXISTS #AGENT_AUDIOS_TIME
	DROP TABLE IF EXISTS #RESULTS_SUMMARY
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
