SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveBatchScheduleExecution] (
	@VCC VARCHAR(50) = NULL,
	@Id VARCHAR(50) = NULL,
	@BatchScheduleId VARCHAR(50) = NULL, 
	@BatchId VARCHAR(50) = NULL,
	@ExecutionDate DATETIME = NULL,
	@Order VARCHAR(50) = NULL,
	@Status VARCHAR(50) = NULL,
	@UserId VARCHAR(50) = NULL
)
AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE REGISTRAR LOS ERRORES QUE PUEDEN OCURRIR DURANTE LA EJECUCION DE LAS COLAS
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
		-- VALIDAMOS QUE TENEMOS LOS PARAMETROS NECESARIOS
		IF @VCC IS NULL 
			OR @Id IS NULL 
			OR @BatchScheduleId IS NULL 
			-- LANZAMOS MENSAJE AVISANDO QUE FALTA ACCOUNTID
			RAISERROR('DATABASE_ERROR_MISSING_PARAMETERS', 16, 16)

		--
		UPDATE [dbo].[BatchScheduleExecution]
			SET [BatchId] = @BatchId,
				[Status] = @Status,
				[LastModifiedDate] = GETUTCDATE(),
				[LastModifiedByUserId] = @UserId,
				[TimeStamp] = GETUTCDATE()
			WHERE [VCC] = @VCC
				AND [Id] = @Id

		-- SI NO SE ACTUALIZO NINGUN REGISTRO, LO INSERTAMOS
		IF @@ROWCOUNT = 0
		BEGIN
			--select * from BatchSchedule (nolock)
			-- INSERTAMOS EL REGISTRO EN LA TABLA DE ERROR
			INSERT INTO [dbo].[BatchScheduleExecution] (
					[VCC], [Id], [BatchScheduleId], [BatchId], [ExecutionDate], 
					[Order], [Status], [CreatedDate], [CreatedByUserId], [TimeStamp]
				) VALUES (
					@VCC, @Id, @BatchScheduleId, @BatchId, @ExecutionDate, 
					@Order, @Status, GETUTCDATE(), @UserId, GETUTCDATE()
				)
		END

		-- EN CASO EL STATUS SEA BATCH CREADO O INICIADO, ACTUALIZAMOS EL REGISTRO DE LA AGENDA
		IF @Status = 'REGISTERED_ON_REDIS' OR @Status = 'BATCH_CREATED' OR @Status = 'BATCH_STARTED'
			UPDATE BS 
				SET BS.[LastExecutionDate] = BSE.[ExecutionDate],
					BS.[Status] = 'RUNNING',
					BS.[LastModifiedDate] = GETUTCDATE(),
					BS.[LastModifiedByUserId] = @UserId,
					BS.[TimeStamp] = GETUTCDATE()
				FROM [dbo].[BatchSchedule] BS 
				INNER JOIN [dbo].[BatchScheduleExecution] BSE
					ON BSE.[VCC] = BS.[VCC]	
						AND BSE.[BatchScheduleId] = BS.[Id]
				WHERE BS.[VCC] = @VCC 
					AND BSE.[Id] = @Id
						

	END TRY
    BEGIN CATCH
        --IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

	SET XACT_ABORT, NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
