SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

CREATE PROCEDURE [dbo].[SaveDictionaryWord] (
	@VCC					VARCHAR(50) = NULL,
	@DictionaryId			VARCHAR(50) = NULL,
	@ID						VARCHAR(50) = NULL,
	@Word					VARCHAR(50) = NULL,
	@SaveUser				VARCHAR(50) = NULL
) AS
BEGIN
	SET NOCOUNT ON
	
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR LOS SALDOS DE LAS CARTERAS DEL VCC
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
		-- VERIFICAMOS SI EL ID EXISTE EN LOS PARAMETROS DE ENTRADA
		IF @ID IS NULL
		BEGIN
			-- VALIDAMOS SI LA PALABRA YA EXISTE EN LA TABLA
			IF EXISTS (
				SELECT * 
					FROM [dbo].[DictionaryWord] (NOLOCK) 
					WHERE [VCC] = @VCC 
						AND [DictionaryId] = @DictionaryId
						AND [Word] = @Word
			)
				RAISERROR('DATABASE_ERROR_DICTIONARY_WORD_ALREADY_EXISTS', 16, 16)

			-- VALIDAMOS SI EL DICCIONARIO A ALCANZO EL LIMITE DE 1000 PALABRAS
			IF 1000 <= (
				SELECT COUNT(*) 
					FROM [dbo].[DictionaryWord] (NOLOCK) 
					WHERE [VCC] = @VCC 
						AND [DictionaryId] = @DictionaryId
			)
				RAISERROR('DATABASE_ERROR_DICTIONARY_ALREADY_HAVE_1000_WORDS', 16, 16)
			
			-- INSERTAMOS LA NUEVA PALABRA
			INSERT INTO [dbo].[DictionaryWord] (
					[VCC], [DictionaryId], [ID], [Word], [CreatedDate], [CreatedByUserId], 
					[TimeStamp]
				)
				VALUES (
					@VCC, @DictionaryId, NEWID(), @Word, GETUTCDATE(), @SaveUser,
					GETUTCDATE()
				)
		END
		ELSE
		BEGIN
			-- ACTUALIZAMOS LA PALABRA POR EL ID
			UPDATE [dbo].[DictionaryWord]
				SET [Word] = @Word,
					[LastModifiedByUserId] = @SaveUser,
					[LastModifiedDate] = GETUTCDATE(),
					[TimeStamp] = GETUTCDATE()
				WHERE [VCC] = @VCC
					AND [DictionaryId] = @DictionaryId
					AND [ID] = @ID
		END
		
		-- RETORNAMOS RESULTADO 1
		SELECT 1 'result'

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH
	
	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
