SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveEvaluationBaseResult] (
	@VCC VARCHAR(50) = NULL,
	@EVALUATION_PLAN_ID VARCHAR(100) = NULL,
	@ABILITY_TYPE VARCHAR(50) = NULL
) AS

BEGIN
	-- ALERTA: NO ESTA PROBBADO PARA PLAN MANUAL

	-- ESTE SP INSERTA LOS RESULT BASE DE UN PLAN AUTOMATICO/MIXTO
	SET NOCOUNT ON

	-- SETTEAMOS EL TYPE ABILITY COMO AUTOMATIC PO DEFECTO
	IF @ABILITY_TYPE IS NULL SET @ABILITY_TYPE = 'AUTOMATIC'

	-- EMULAMOS UNA NUEVA TABLA VARIABLE PARA OBTENER LOS REGISTROS UNICOS QUE INSERTAREMOS EN LA TABLA EVALUATION_RESULT
	--  ORDENANDO POR SECTION Y PART

	DROP TABLE IF EXISTS #TMP_TRANSCRIBED_CALL_INTERACTIONS
	DROP TABLE IF EXISTS #TMP_RESULTS_FOR_EVALUATION 
	DROP TABLE IF EXISTS #TMP_EVALUATIONS_RESULTS

	-- CREAMOS UNA TABLA TEMPORAL PARA ALMACENAR LAS INTERACCIONES CALL YA TRANSRITAS
	SELECT A.[ID]
		INTO #TMP_TRANSCRIBED_CALL_INTERACTIONS
		FROM [dbo].[Audio] A (NOLOCK)
		INNER JOIN [dbo].[ProcessAudio] PA (NOLOCK)
			ON PA.[VCC] = A.[VCC]
				AND PA.[AudioId] = A.[ID] 
		INNER JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
			ON EP.[Instance] = A.[VCC]
				AND EP.[ProcessId] = PA.[ProcessId]
		WHERE A.[VCC] = @VCC 
			AND A.[IsTranscribed] = 1
			AND EP.[ID] = @EVALUATION_PLAN_ID

	-- SELECT * FROM #TMP_TRANSCRIBED_CALL_INTERACTIONS
	
	-- CREAMOS UNA TABLA TEMPORAL PARA ALMACENAR LAS INTERACCIONES TEXT YA TRANSRITAS
	SELECT AI.[ID]
		INTO #TMP_TRANSCRIBED_TEXT_INTERACTIONS
		FROM [dbo].[AsyncInteraction] AI (NOLOCK)
		INNER JOIN [dbo].[ProcessAsyncInteraction] PAI (NOLOCK)
			ON PAI.[VCC] = AI.[VCC]
				AND PAI.[InteractionId] = AI.[ID]
				AND PAI.[FormattedEndDate] = AI.[Folder]
		INNER JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
			ON EP.[Instance] = AI.[VCC]
				AND EP.[ProcessId] = PAI.[ProcessId]
		WHERE AI.[VCC] = @VCC 
			AND AI.[IsTranscribed] = 1
			AND EP.[ID] = @EVALUATION_PLAN_ID

	-- CREAMOS UNA TABLA TEMPORAL PARA ALMACENAR LOS EVALUATION_RESULT YA REGISTRADOS
	SELECT ER.*
		INTO #TMP_EVALUATIONS_RESULTS
		FROM [dbo].[EvaluationResult] ER (NOLOCK) 
		WHERE ER.[Instance] = @VCC 
			AND ER.[IdEvaluationPlan] = @EVALUATION_PLAN_ID

	-- SELECT * FROM #TMP_TRANSCRIBED_TEXT_INTERACTIONS
	
	-- GENERAMOS REGISTROS PARA TODAS LAS INTERACCIONES
	;WITH VALID_INTERACTIONS_CTE AS (
		SELECT [ID] FROM #TMP_TRANSCRIBED_CALL_INTERACTIONS
		UNION ALL
		SELECT [ID] FROM #TMP_TRANSCRIBED_TEXT_INTERACTIONS
	)
	, INTERACTIONS_ALL_CTE AS (
		SELECT IEP.[Instance],
				IEP.[IdEvaluationPlan],
				IEP.[IdCampaign],
				IEP.[IdInteraction],
				IEP.[IdAgent],
				IEP.[InteractionDateStart],
				IEP.[InteractionDateEnd],
				IEP.[InteractionType],
				IEP.[ContactId],
				RN = ROW_NUMBER() OVER(PARTITION BY IEP.[Instance],
													IEP.[IdEvaluationPlan],				
													IEP.[IdCampaign],
													IEP.[IdInteraction],
													IEP.[IdAgent],
													IEP.[InteractionDateStart],
													IEP.[InteractionDateEnd],
													IEP.[InteractionType],
													IEP.[ContactId]
										ORDER BY IEP.[InteractionSection] ASC, IEP.[InteractionPart] ASC)
			FROM [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
			INNER JOIN VALID_INTERACTIONS_CTE IA_CTE 
				ON IA_CTE.[ID] = IEP.[IdInteraction]
			WHERE IEP.[Instance] = @VCC
				AND IEP.[IdEvaluationPlan] = @EVALUATION_PLAN_ID
	)  -- SELECT * FROM INTERACTIONS_ALL_CTE RETURN
	, INTERACTIONS_SUMMARY_CTE AS (
		SELECT I_A_CTE.*
			FROM INTERACTIONS_ALL_CTE I_A_CTE
			WHERE I_A_CTE.RN = 1
	) -- SELECT * FROM INTERACTIONS_SUMMARY_CTE RETURN
	, INTERACTIONS_FIRST_FILTER_CTE AS (
		SELECT * 
			FROM INTERACTIONS_SUMMARY_CTE IS_CTE
			WHERE (
					IS_CTE.[InteractionType] = 'CALL' 
					AND 
					IS_CTE.[IdInteraction] IN (
						SELECT [ID] FROM #TMP_TRANSCRIBED_CALL_INTERACTIONS
					)
				)
				OR
				IS_CTE.[InteractionType] <> 'CALL' 
	)  -- SELECT * FROM INTERACTIONS_FIRST_FILTER_CTE RETURN /*
	-- TODO: EL SECOND_FILTER ES SOBRE LAS INTERACCIONES TEXT TRANSCRITAS
	, MAIN_CTE AS (
		SELECT EP.[Instance]										'vcc',
				EP.[ID]												'IdEvaluationPlan',
				EP.[IdEvaluationFormat]								'IdEvaluationFormat',
				HEF.[MinQualificationAcceptable]					'MinQualificationAcceptable',
				IFF_CTE.[IdInteraction]								'IdInteraction',
				IFF_CTE.[IdCampaign]								'IdCampaign',
				IFF_CTE.[IdAgent]									'IdAgent',
				EP.[ProcessId]										'ProcessId',
				HAEF.[IdGroup]										'IdGroup',
				HAEF.[IdSubGroup]									'IdSubGroup',
				HAEF.[IdAbility]									'IdAbility',
				HAEF.[Score]										'MaxAbilityScore',
				HS.[Value]											'AbilityInnerScore',
				HS.[Description]									'ScoreDescription',
				IFF_CTE.[IdCampaign]								'CampaignDescription',
				IFF_CTE.[IdAgent]									'AgentDescription',
				IFF_CTE.[InteractionDateStart]						'InteractionDateini',
				IFF_CTE.[InteractionDateEnd]						'InteractionDatefin',
				IFF_CTE.[ContactId]									'InteractionAddress',
				HAEF.[Order]										'AbilityOrder',
				IFF_CTE.[InteractionType]							'InteractionType',
				HAEF.[ID]											'IdAbilityEvaluationFormat',
				HAEF.[Conditional]									'Conditional'
			FROM [dbo].[EvaluationPlan] EP (NOLOCK)
			INNER JOIN [dbo].[HistoricEvaluationFormat] HEF (NOLOCK)
				ON HEF.[VCC] = EP.[Instance]
					AND HEF.[EvaluationPlanId] = EP.[ID]
			INNER JOIN [dbo].[HistoricAbilityEvaluationFormat] HAEF (NOLOCK)
				ON HAEF.[VCC] = EP.[Instance]
					AND HAEF.[EvaluationPlanId] = EP.[ID]
			INNER JOIN [dbo].[HistoricAbility] HA (NOLOCK)
				ON HA.[VCC] = EP.[Instance]
					AND HA.[EvaluationPlanId] = EP.[ID]
					AND HA.[ID] = HAEF.[IdAbility]
			INNER JOIN [dbo].[HistoricScore] HS (NOLOCK)
				ON HS.[VCC] = EP.[Instance]
					AND HS.[EvaluationPlanId] = EP.[ID]
					AND HS.[IdAbility] = HA.[ID]
					AND HS.[Value] = IIF(HA.[IsNegative] = 1, 100, 0)
			INNER JOIN INTERACTIONS_FIRST_FILTER_CTE IFF_CTE 
				ON IFF_CTE.[Instance] = EP.[Instance]
					AND IFF_CTE.[IdEvaluationPlan] = EP.[ID]
			LEFT JOIN #TMP_EVALUATIONS_RESULTS TMP_ER 
				ON TMP_ER.[Instance] = EP.[Instance]
					AND TMP_ER.[IdEvaluationPlan] = EP.[ID]
					AND TMP_ER.[IdInteraction] = IFF_CTE.[IdInteraction]
					AND TMP_ER.[IdAbility] = HA.[ID]
					AND TMP_ER.[IdAgent] = IFF_CTE.[IdAgent]
					AND TMP_ER.[IdAbilityEvaluationFormat] = HAEF.[ID]
			WHERE TMP_ER.[Instance] IS NULL
				AND EP.[Instance] = @VCC
				AND IFF_CTE.[IdInteraction] IS NOT NULL
			    AND EP.[ID] = @EVALUATION_PLAN_ID
				AND (
					(
						@ABILITY_TYPE = 'AUTOMATIC' 
						AND 
						HA.[Type] = 'MANUAL_AUTOMATIC'
					)
					OR
					(
						@ABILITY_TYPE = 'MANUAL' 
						AND 
						HA.[Type] = 'MANUAL'
					)
				)
	)  -- SELECT * FROM MAIN_CTE RETURN
	
		
	-- INSERTAMOS TODOS LOS REGISTROS EN UNA TABLA TEMPORAL
	SELECT * -- 0 'DEBUG'
		INTO #TMP_RESULTS_FOR_EVALUATION
		FROM MAIN_CTE
	
	-- ELIMINAMOS LOS REGISTROS PARA INTERACCIONES TEXT NO TRANSCRITAS
	/*
	DELETE FROM TMP_RESULTS_FOR_EVALUATION
		WHERE [InteractionType] <> 'CALL' 
			AND [IdInteraction] NOT IN (
				SELECT [ID] FROM #TMP_NOT_TRANSCRIBED_TEXT_INTERACTIONS
			)
			*/

	-- TENEMOS LOS REGISTROS PARA INSERTAR EN EVALUATIONRESULT
	-- SELECT * FROM #TMP_RESULTS_FOR_EVALUATION RETURN

	-- INSERTAMOS SOLO LAS PRIMERAS OCURRENCIAS PARA CADA INTERACCION
	INSERT INTO [dbo].[EvaluationResult] (
		[Instance], [IdCampaign], [IdInteraction],
		[IdAgent], [IdEvaluationFormat], [MinQualificationAcceptable],
		[IdGroup], [IdSubGroup], [IdAbility],
		[Qualification], [OwnScore], [AbilityInnerScore],
		[IdEvaluator], [EvaluationDate], [MaxAbilityScore],
		[IdEvaluationPlan], [CreatedDate], [CreatedByUserId],
		[LastModifiedDate], [LastModifiedByUserId], [TimeStamp],
		[Comment], [NotApplies], [ScoreDescription],
		[CampaignDescription], [AgentDescription], [InteractionDateini],
		[InteractionDatefin], [InteractionAddress], [AbilityOrder],
		[InteractionType], [IdAbilityEvaluationFormat],  [IdAuditor],
		[Conditional]
	)
		SELECT TMP_RFE.[vcc], TMP_RFE.[IdCampaign], TMP_RFE.[IdInteraction],
				TMP_RFE.[IdAgent], TMP_RFE.[IdEvaluationFormat], TMP_RFE.[MinQualificationAcceptable],
				TMP_RFE.[IdGroup], TMP_RFE.[IdSubGroup], TMP_RFE.[IdAbility],
				(TMP_RFE.[MaxAbilityScore] * TMP_RFE.[AbilityInnerScore] / 100), (TMP_RFE.[MaxAbilityScore] * TMP_RFE.[AbilityInnerScore] / 100), TMP_RFE.[AbilityInnerScore],
				'SPEECH', GETUTCDATE(), TMP_RFE.[MaxAbilityScore],
				TMP_RFE.[IdEvaluationPlan], GETUTCDATE(), 'SPEECH',
				--'100', TMP_RFE.[IdEvaluationPlan], GETUTCDATE(),
				GETUTCDATE(), 'SPEECH', GETUTCDATE(),
				NULL, '0', TMP_RFE.[ScoreDescription],
				TMP_RFE.[CampaignDescription], TMP_RFE.[AgentDescription], TMP_RFE.[InteractionDateini],
				TMP_RFE.[InteractionDatefin], TMP_RFE.[InteractionAddress], TMP_RFE.[AbilityOrder],
				TMP_RFE.[InteractionType], TMP_RFE.[IdAbilityEvaluationFormat], 'SPEECH',
				TMP_RFE.[Conditional]
			FROM #TMP_RESULTS_FOR_EVALUATION TMP_RFE

	-- ELIMINAMOS LAS TABLAS TEMPORALES
	DROP TABLE #TMP_TRANSCRIBED_CALL_INTERACTIONS
	DROP TABLE #TMP_RESULTS_FOR_EVALUATION
	DROP TABLE #TMP_EVALUATIONS_RESULTS

	SET NOCOUNT OFF

END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
