SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveI6InteractionAsyncInteraction] (
	@VCC					VARCHAR(50) = NULL,
	@InteractionId			VARCHAR(250) = NULL,
	@Folder					VARCHAR(50) = NULL,
	@ExportedContent		VARCHAR(250) = NULL,
	@Channel				VARCHAR(25) = NULL,
	@StartDate				DATETIME = NULL,
	@EndDate				DATETIME = NULL,
	@DurationTime			INT = NULL,
	@AgentId				VARCHAR(50) = NULL,
	@Campaign				VARCHAR(100) = NULL,
	@Account				VARCHAR(50) = NULL,
	@Origin					VARCHAR(25) = NULL,
	@DispositionTreePath	VARCHAR(50) = NULL,
	@DispositionCode		VARCHAR(50) = NULL,
	@ContactId				VARCHAR(50) = NULL,
	@ContactName			VARCHAR(300) = NULL,
	@ContactAddress			VARCHAR(300) = NULL,
	@TotalMessages			INT = NULL,
	@ProcessId				VARCHAR(50) = NULL
) AS
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR UNA INTERACCION DE I6
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY
	--select * from AsyncInteraction (nolock)
	--select * from InteractionEvaluationPlan (nolock)
		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		-- DECLARAMOS UNA VARIABLE PARA AYUDAR A LA ACTUALIZACION
		DECLARE @BD_AI_ID VARCHAR(250) = NULL
		DECLARE @NEED_I6_UPDATE BIT = 1
		DECLARE @AGENT_NAME VARCHAR(100) = NULL -- NOMBRE DEL AGENTE
		DECLARE @EVALUATION_PLAN_ID VARCHAR(50) = NULL -- ID DEL PLAN EN CASO EXISTA

		-- ASIGNAMOS EL VALOR ACTUALIZADO DEL FLAG
		SELECT @BD_AI_ID = AI.[Id],
				@NEED_I6_UPDATE = AI.[NeedI6Update]
			FROM [dbo].[AsyncInteraction] AI (NOLOCK)
			WHERE AI.[VCC] = @VCC
				AND AI.[ID] = @InteractionId
				AND AI.[Folder] = @Folder

		-- BUSCAMOS EL ID DEL PLAN EN CASO EXISTA
		SELECT @EVALUATION_PLAN_ID = EP.[ID]
			FROM [dbo].[EvaluationPlan] EP (NOLOCK)
			WHERE EP.[Instance] = @VCC 
				AND EP.[ProcessId] = @ProcessId
				AND EP.[Status] < 5
		
		-- ASIGNAMOS A LA VARIABLE EL NOMBRE DEL AGENTE
		SELECT @AGENT_NAME = A.[Name]
			FROM [dbo].[Agent] A (NOLOCK)
			WHERE A.[VCC] = @VCC
				AND A.[AgentId] = @AgentId

		-- ASIGNAMOS EL NAME EN CASO SEA NULL
		SET @AGENT_NAME = IIF((@AgentId IS NOT NULL AND @AgentId <> ''), @AgentId, NULL)
		
		-- EN CASO REQUIERAN ACTUALIZACION POR UNICA VEZ
		IF @NEED_I6_UPDATE = 1 AND @ProcessId IS NOT NULL
		BEGIN
			print @BD_AI_ID
			-- VERIFICAMOS IS EXISTE
			IF @BD_AI_ID IS NOT NULL
			BEGIN
				PRINT 'UPDATE'
				-- ACTUALIZAMOS LA TABLA DE LA INTERACCION EN LA TABLA ASYNC_INTERACTION
				UPDATE [dbo].[AsyncInteraction]
					SET [ExportedContent] = @ExportedContent,
						[Campaign]= @Campaign,
						[StartDate] = @StartDate,
						[EndDate] = @EndDate,
						[Account] = @Account,
						[Initiative] = @Origin,
						[LastTakedAgent] = @AgentId,
						[LastTakedAgentName] = @AGENT_NAME,
						[DurationTime] = @DurationTime,
						[DispositionTreePath] = @DispositionTreePath,
						[DispositionCode] = @DispositionCode,
						[ContactId] = @ContactId,
						[ContactName] = @ContactName,
						[ContactAddress] = @ContactAddress,
						[TotalMessages] = @TotalMessages,
						[NeedI6Update] = 0,
						[IsProcess] = NULL,
						[IsCancel] = NULL,
						[LastModifiedByUserId] = 'system',
						[LastModifiedDate] = GETUTCDATE(),
						[TimeStamp] = GETUTCDATE()
					WHERE [VCC] = @VCC
						AND [ID] = @InteractionId
						AND [Folder] = @Folder
						AND [NeedI6Update] = 1
						AND (
							[IsTranscribed] IS NULL 
							OR
							[IsTranscribed] <> 1
						)

				-- ACTUALIZAMOS EL CAMPAIGN Y ENDDATE EN LA INTERACCION EN LA TABLA PROCESSAUDIO
				UPDATE [dbo].[ProcessAsyncInteraction]
					SET [Campaign] = @Campaign,
						[EndDate] = @EndDate,
						[LastModifiedByUserId] = 'system',
						[LastModifiedDate] = GETUTCDATE(),
						[TimeStamp] = GETUTCDATE()
					WHERE [VCC] = @VCC
						AND [InteractionId] = @InteractionId
						AND [FormattedEndDate] = @Folder 
						AND [ProcessId] = @ProcessId
			END
			ELSE
			BEGIN
				-- SELECT * FROM AsyncInteraction (NOLOCK)
				-- SELECT * FROM ProcessAsyncInteraction (NOLOCK)
				-- INSERTAMOS EL REGISTRO EN ASYNC_INTERACTION
				INSERT INTO [dbo].[AsyncInteraction] (
					[VCC], [ID], [Folder], [ExportedContent], [Channel], 
					[Campaign], [StartDate], [EndDate], [Account], [Initiative], 
					[ContactId], [ContactName], [ContactAddress], [LastTakedAgent], [LastTakedAgentName], 
					[TotalMessages], [DurationTime], [AttentionTime], [DispositionCode], [DispositionTreePath], 
					[NeedI6Update], [LastModifiedByUserId], [LastModifiedDate], [TimeStamp]
				) VALUES (
					@VCC, @InteractionId, @Folder, @ExportedContent, @Channel, 
					@Campaign, @StartDate, @EndDate, @Account, @Origin, 
					@ContactId, @ContactName, @ContactAddress, @AgentId, @AGENT_NAME, 
					@TotalMessages, @DurationTime, @DurationTime, @DispositionCode, @DispositionTreePath, 
					0, 'system', GETUTCDATE(), GETUTCDATE()
				)

				-- INSERTAMOS EL REGISTRO EN PROCESS_ASYNC_INTERACTION
				INSERT INTO [dbo].[ProcessAsyncInteraction] (
					[VCC], [ProcessId], [InteractionType], [Campaign], [InteractionId],
					[EndDate], [FormattedEndDate], [CreatedDate], [CreatedByUserId], [TimeStamp]
				) VALUES (
					@VCC, @ProcessId, @Channel, @Campaign, @InteractionId, 
					@EndDate, @Folder, GETUTCDATE(), 'system', GETUTCDATE()
				)
			END
		END

		-- ACTUALIZAMOS LOS REGISTROS DE PLAN
		EXEC [dbo].[SaveI6InteractionAsyncInteractionForEvaluationPlan] 
			@VCC, @InteractionId, @Folder, @ExportedContent, @Channel,  
			@StartDate, @EndDate, @DurationTime, @AgentId, @AGENT_NAME, 
			@Campaign, @Account, @Origin, @DispositionTreePath, @DispositionCode, 
			@ContactId, @ContactName, @ContactAddress, @TotalMessages, @EVALUATION_PLAN_ID

		-- RETORNAMOS RESULTADO 1
		SELECT 1 'result'

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

	SET XACT_ABORT, NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
