SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveI6InteractionAudio] (
	@VCC					VARCHAR(50) = NULL,
	@InteractionId			VARCHAR(250) = NULL,
	@OldSection				INT = NULL,
	@OldPart				INT = NULL,
	@NewSection				INT = NULL,
	@NewPart				INT = NULL,
	@StartDate				DATETIME = NULL,
	@EndDate				DATETIME = NULL,
	@DurationTime			INT = NULL,
	@AgentId				VARCHAR(50) = NULL,
	@Format					VARCHAR(10) = NULL,
	@Campaign				VARCHAR(100) = NULL,
	@Origin					VARCHAR(25) = NULL,
	@Category1				VARCHAR(100) = NULL,
	@Category2				VARCHAR(100) = NULL,
	@Category3				VARCHAR(100) = NULL,
	@Description			VARCHAR(100) = NULL,
	@Filename				VARCHAR(200) = NULL,
	@ContactId				VARCHAR(50) = NULL,
	@ContactName			VARCHAR(300) = NULL,
	@ContactAddress			VARCHAR(300) = NULL,
	@ProcessId				VARCHAR(50) = NULL
) AS 
BEGIN
	-- ESTE SP TIENE LA FINALIDAD DE ACTUALIZAR UNA INTERACCION DE I6
	SET XACT_ABORT, NOCOUNT ON

	-- INICIAMOS UN TRY PARA EL CASO CAIGA EN CATCH HAGAMOS ROLLBACK
    BEGIN TRY

		-- INICIAMOS LA TRANSACCION
		BEGIN TRANSACTION

		-- DECLARAMOS UNA VARIABLE PARA AYUDAR A LA ACTUALIZACION
		DECLARE @BD_A_ID VARCHAR(250) = NULL
		DECLARE @NEED_I6_UPDATE BIT = 1
		DECLARE @AGENT_NAME VARCHAR(100) = NULL -- NOMBRE DEL AGENTE
		DECLARE @EVALUATION_PLAN_ID VARCHAR(50) = NULL -- ID DEL PLAN EN CASO EXISTA

		-- ASIGNAMOS EL VALOR ACTUALIZADO DEL FLAG
		SELECT @BD_A_ID = A.[ID],
				@NEED_I6_UPDATE = A.[NeedI6Update]
			FROM [dbo].[Audio] A (NOLOCK)
			WHERE A.[VCC] = @VCC
				AND A.[ID] = @InteractionId
				AND A.[Section] = @OldSection
				AND A.[Part] = @OldPart

		-- BUSCAMOS EL ID DEL PLAN EN CASO EXISTA
		SELECT @EVALUATION_PLAN_ID = EP.[ID]
			FROM [dbo].[EvaluationPlan] EP (NOLOCK)
			WHERE EP.[Instance] = @VCC 
				AND EP.[ProcessId] = @ProcessId
				AND EP.[Status] = 1

		-- ASIGNAMOS A LA VARIABLE EL NOMBRE DEL AGENTE
		SELECT @AGENT_NAME = A.[Name]
			FROM [dbo].[Agent] A (NOLOCK)
			WHERE A.[VCC] = @VCC
				AND A.[AgentId] = @AgentId

		-- EN CASO REQUIERAN ACTUALIZACION POR UNICA VEZ
		IF @NEED_I6_UPDATE = 1 AND @ProcessId IS NOT NULL
		BEGIN
			-- VERIFICAMOS IS EXISTE
			IF @BD_A_ID IS NOT NULL
			BEGIN
				-- ACTUALIZAMOS EL SECTION Y PART EN LA INTERACCION EN LA TABLA AUDIO
				UPDATE [dbo].[Audio] 
					SET [Section] = @NewSection,
						[Part] = @NewPart,
						[Agent] = @AgentId,
						[AgentName] = @AGENT_NAME,
						[Format] = @Format,
						[Campaign]= @Campaign,
						[Type] = @Origin,
						[Category1] = @Category1,
						[Category2] = @Category2,
						[Category3] = @Category3,
						[Description] = @Description,
						[FileName] = @Filename,
						[ContactId] = @ContactId,
						[ContactName] = @ContactName,
						[ContactAddress] = @ContactAddress,
						[NeedI6Update] = 0,
						[IsProcess] = NULL,
						[IsCancel] = NULL,
						[LastModifiedByUserId] = 'system',
						[LastModifiedDate] = GETUTCDATE(),
						[TimeStamp] = GETUTCDATE()
					WHERE [VCC] = @VCC
						AND [ID] = @InteractionId
						AND [Section] = @OldSection
						AND [Part] = @OldPart
						AND [NeedI6Update] = 1
						AND (
							[IsTranscribed] IS NULL 
							OR
							[IsTranscribed] <> 1
						)

				IF @@ROWCOUNT > 0
				BEGIN
					-- ACTUALIZAMOS EL SECTION Y PART EN LA INTERACCION EN LA TABLA PROCESSAUDIO
					UPDATE [dbo].[ProcessAudio]
						SET [Section] = @NewSection,
							[Part] = @NewPart
						WHERE [VCC] = @VCC
							AND [AudioId] = @InteractionId
							AND [Section] = @OldSection
							AND [Part] = @OldPart 
				END
			END
			ELSE
			BEGIN
				-- INSERTAMOS EL NUEVO AUDIO CUANDO NO RECIBIMOS SECTION Y PART
				INSERT INTO [dbo].[Audio] (
					[VCC], [Id], [Section], [Part], [Channel],
					[Number], [Format], [DurationTime], [Category1], [Category2],
					[Category3], [Description], [Filename], [Campaign], [Agent], 
					[AgentName], [Type], [StartDate], [EndDate], [ContactId], 
					[ContactName], [ContactAddress], [NeedI6Update], [CreatedByUserId], [CreatedDate], 
					[TimeStamp]
				)
				VALUES (
					@VCC, @InteractionId, @NewSection, @NewPart, 'CALL',
					@ContactAddress, @Format, @DurationTime, @Category1, @Category2,
					@Category3, @Description, @Filename, @Campaign, @AgentId, 
					@AGENT_NAME, @Origin, @StartDate, @EndDate, @ContactId, 
					@ContactName, @ContactAddress, 0, 'system', GETUTCDATE(), 
					GETUTCDATE()
				)

				INSERT INTO [dbo].[ProcessAudio] (
					[VCC], [ProcessId], [AudioId], [Section], [Part],
					[CreatedDate], [CreatedByUserId], [TimeStamp]
				)
				VALUES (
					@VCC, @ProcessId, @InteractionId, @NewSection, @NewPart,
					GETUTCDATE(), 'system', GETUTCDATE()
				)
			END 
		END

		-- ACTUALIZAMOS LOS REGISTROS DE PLAN
		EXEC [dbo].[SaveI6InteractionSyncInteractionForEvaluationPlan] 
			@VCC, @InteractionId, @OldSection, @OldPart, @NewSection,
			@NewPart, @StartDate, @EndDate, @DurationTime, @Format,
			@AgentId, @AGENT_NAME, @Campaign, @Origin, @Category1,
			@Category2, @Category3, @Description, @ContactId, @ContactName,
			@ContactAddress, @EVALUATION_PLAN_ID
			
		-- RETORNAMOS RESULTADO 1
		SELECT 1 'result'

		-- COMPLETAMOS LA TRANSACCION
		COMMIT TRANSACTION

    END TRY
    BEGIN CATCH
        IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION

        DECLARE @Message nvarchar(2048) = ERROR_MESSAGE()
        DECLARE @Severity integer = ERROR_SEVERITY()
        DECLARE @State integer = ERROR_STATE()

        RAISERROR(@Message, @Severity, @State)
        RETURN -1
    END CATCH

	SET XACT_ABORT, NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
