SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO
CREATE PROCEDURE [dbo].[SaveInteractionEvaluationPlanSample] (
	@VCC VARCHAR(50) = NULL,
	@EvaluationPlanId VARCHAR(50) = NULL,
	@EvaluatePercentage DECIMAL(18, 2) = NULL
) AS
BEGIN
	SET NOCOUNT ON

	-- ESTE SP REDUCE LAS INTERACCIONES DE UN PLAN AL PORCENTAJE DADO EN EL INPUT
	/*
	DECLARE @VCC VARCHAR(50) = 'occ_test'
	DECLARE @EvaluationPlanId VARCHAR(50)  = '50c7da87-c79d-4f05-b66d-4f94d76f8e55' 
	DECLARE @EvaluatePercentage DECIMAL(18, 2) = 0.20
	*/

	-- DECLARAMOS UNA VARIABLE PARA ALMACENAR EL NUMERO TOTAL DE INTERACCIONES
	DECLARE @CHANNEL_COUNTER INT = 0
	DECLARE @CURRENT_TOTAL_INTERACTIONS INT = 0
	DECLARE @NEW_TOTAL_INTERACTIONS DECIMAL(18, 2) = 0

	-- ALMACENAMOS EL TOTAL DE INTERACCIONES QUE EXISTEN ACTUALMENTE
	SELECT @CURRENT_TOTAL_INTERACTIONS = COUNT(DISTINCT IEP.[IdInteraction])
		FROM [InteractionEvaluationPlan] IEP (NOLOCK)
		WHERE IEP.[Instance] = @VCC
			AND IEP.[IdEvaluationPlan] = @EvaluationPlanId

	-- SETTEAMOS EL NUEVO TOTAL DE INTERACCIONES
	SET @NEW_TOTAL_INTERACTIONS =  CASE WHEN ((@CURRENT_TOTAL_INTERACTIONS * @EvaluatePercentage) % 1) >= 0.5 
									THEN CEILING(@CURRENT_TOTAL_INTERACTIONS * @EvaluatePercentage) 
									ELSE FLOOR(@CURRENT_TOTAL_INTERACTIONS * @EvaluatePercentage) 
									END

	-- DECLARAMOS UNA VARIABLE PARA ALMACENAR LOS CANALES Y NUMERO DE INTERACCIONES 
	DECLARE MY_CURSOR CURSOR 
	LOCAL STATIC READ_ONLY FORWARD_ONLY -- SCROLL 
		FOR 
		-- CONSULTAMOS LA CANTIDAD DE INTERACCIONES QUE EXISTEN EN EL PLAN
		SELECT IEP.[InteractionType]													'CHANNEL',
				COUNT(DISTINCT IEP.[IdInteraction])										'CHANNEL_INTERACTIONS',
				CEILING(COUNT(DISTINCT IEP.[IdInteraction]) * @EvaluatePercentage)		'NEW_CHANNEL_INTERACTIONS'
			FROM [InteractionEvaluationPlan] IEP (NOLOCK)
			WHERE IEP.[Instance] = @VCC
				AND IEP.[IdEvaluationPlan] = @EvaluationPlanId
			GROUP BY IEP.[InteractionType]
			ORDER BY COUNT(DISTINCT IEP.[IdInteraction]) ASC

	-- DECLARARAMOS UNA VARIABLE PARA INDICAR EL BACTH ACTUAL QUE ESTAMOS RECORRIENDO
	DECLARE @CURRENT_CHANNEL VARCHAR(50) = ''
	DECLARE @CURRENT_CHANNEL_INTERACTIONS INT = 0
	DECLARE @NEW_CHANNEL_INTERACTIONS INT = 0
	
	-- ABRIMOS Y RECORREMOS EL ARRAY DE CANALES
	OPEN MY_CURSOR
	FETCH NEXT FROM MY_CURSOR INTO @CURRENT_CHANNEL, @CURRENT_CHANNEL_INTERACTIONS, @NEW_CHANNEL_INTERACTIONS
	WHILE @@FETCH_STATUS = 0
	BEGIN 
		-- AUMENTAMOS EL CONTADOR DE CANALES
		SET @CHANNEL_COUNTER = @CHANNEL_COUNTER + 1

		-- VERIFICAMOS SI AUN QUEDAN CUPOS DISPONIBLES
		IF @NEW_TOTAL_INTERACTIONS > 0
		BEGIN
			-- VERIFICMAOS SI LA CANTIDAD PARA EL CANAL ES MAYOR A LA CANTIDAD DE CUPOS DISPONIBLES
			IF @NEW_TOTAL_INTERACTIONS <= @NEW_CHANNEL_INTERACTIONS 
			BEGIN
				-- SI LOS CUPOS DISPONIBLES SON MENORES A LA CANTIDAD PARA EL CANAL ACTUAL, LA CANTIDAD DISPONIBLE PASA A SER LA CANTIDAD PARA EL CANAL
				SET @NEW_CHANNEL_INTERACTIONS = @NEW_TOTAL_INTERACTIONS

			END
			ELSE
			BEGIN
				-- SI EL CONTADOR ES IGUAL A @@CURSOR_ROWS  ESTAMOS EN EL ULTIMO ITEM DEL CURSOR
				IF @CHANNEL_COUNTER = @@CURSOR_ROWS 
				BEGIN 
					-- ACTUALIZAMOS LA CANTIDAD PARA EL CANAL ACTUAL
					SET @NEW_CHANNEL_INTERACTIONS = @NEW_TOTAL_INTERACTIONS
					-- SELECT @CURRENT_CHANNEL, '=> AJUSTE =>', @NEW_CHANNEL_INTERACTIONS 
				END
			END
		END
		ELSE
		BEGIN
			-- SI NO QUEDAN CUPOS PENDIENTES, NO SE CONSERVARA NINGUN REGISTROS DEL CANAL
			SET @NEW_CHANNEL_INTERACTIONS = 0
		END
		
		---- ELIMINAMOS LAS INTERACCIONES QUE NO COINCIDEN CON LAS INTERACCIONES DE LA MUESTRA
		DELETE FROM [dbo].[InteractionEvaluationPlan]
			WHERE [Instance] = @VCC
				AND [IdEvaluationPlan] = @EvaluationPlanId
				AND [InteractionType] = @CURRENT_CHANNEL
				AND [IdInteraction] NOT IN (
					-- RETORNAMOS X INTERACCIONES ALEATORIAS POR ID DE INTERACCION
					SELECT TOP (@NEW_CHANNEL_INTERACTIONS) [IEP].[IdInteraction] 
						FROM [dbo].[InteractionEvaluationPlan] IEP (NOLOCK) 
						WHERE IEP.[Instance] = @VCC
							AND IEP.[IdEvaluationPlan] = @EvaluationPlanId
							AND IEP.[InteractionType] = @CURRENT_CHANNEL
						GROUP BY [IEP].[IdInteraction] 
						ORDER BY NEWID()
				)
	
		-- FINALIZAMOS EL BUCLE RESTANDO A LOS CUPOS LA CANTIDAD USADA PARA EL CANAL
		SET @NEW_TOTAL_INTERACTIONS  = @NEW_TOTAL_INTERACTIONS - @NEW_CHANNEL_INTERACTIONS
		
		FETCH NEXT FROM MY_CURSOR INTO @CURRENT_CHANNEL, @CURRENT_CHANNEL_INTERACTIONS, @NEW_CHANNEL_INTERACTIONS
	END
	CLOSE MY_CURSOR
	DEALLOCATE MY_CURSOR

	-- RETORNAMOS UNA RESPUESTA
	SELECT 1 AS 'result'

	SET NOCOUNT OFF
END
GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS OFF 
GO

GO
